# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import Audits, HostsOrdering


class AuditsTestCase(unittest.TestCase):
    def test_create_audit(self):
        request = Audits.create_audit(
            name="foo", policy_id="c1", target_id="t1", scanner_id="s1"
        )

        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>foo</name>"
            b"<usage_type>audit</usage_type>"
            b'<config id="c1"/>'
            b'<target id="t1"/>'
            b'<scanner id="s1"/>'
            b"</create_task>",
        )

    def test_create_audit_with_comment(self):
        request = Audits.create_audit(
            name="foo",
            policy_id="c1",
            target_id="t1",
            scanner_id="s1",
            comment="bar",
        )

        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>foo</name>"
            b"<usage_type>audit</usage_type>"
            b'<config id="c1"/>'
            b'<target id="t1"/>'
            b'<scanner id="s1"/>'
            b"<comment>bar</comment>"
            b"</create_task>",
        )

    def test_create_audit_with_single_alert(self):
        request = Audits.create_audit(
            name="foo",
            policy_id="c1",
            target_id="t1",
            scanner_id="s1",
            alert_ids=["a1"],
        )

        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>foo</name>"
            b"<usage_type>audit</usage_type>"
            b'<config id="c1"/>'
            b'<target id="t1"/>'
            b'<scanner id="s1"/>'
            b'<alert id="a1"/>'
            b"</create_task>",
        )

    def test_create_audit_with_multiple_alerts(self):
        request = Audits.create_audit(
            name="foo",
            policy_id="c1",
            target_id="t1",
            scanner_id="s1",
            alert_ids=["a1", "a2"],
        )

        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>foo</name>"
            b"<usage_type>audit</usage_type>"
            b'<config id="c1"/>'
            b'<target id="t1"/>'
            b'<scanner id="s1"/>'
            b'<alert id="a1"/>'
            b'<alert id="a2"/>'
            b"</create_task>",
        )

    def test_create_audit_with_empty_alert_ids(self):
        request = Audits.create_audit(
            name="foo",
            policy_id="c1",
            target_id="t1",
            scanner_id="s1",
            alert_ids=[],
        )

        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>foo</name>"
            b"<usage_type>audit</usage_type>"
            b'<config id="c1"/>'
            b'<target id="t1"/>'
            b'<scanner id="s1"/>'
            b"</create_task>",
        )

    def test_create_audit_with_alterable(self):
        request = Audits.create_audit(
            name="foo",
            policy_id="c1",
            target_id="t1",
            scanner_id="s1",
            alterable=True,
        )

        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>foo</name>"
            b"<usage_type>audit</usage_type>"
            b'<config id="c1"/>'
            b'<target id="t1"/>'
            b'<scanner id="s1"/>'
            b"<alterable>1</alterable>"
            b"</create_task>",
        )

        request = Audits.create_audit(
            name="foo",
            policy_id="c1",
            target_id="t1",
            scanner_id="s1",
            alterable=False,
        )

        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>foo</name>"
            b"<usage_type>audit</usage_type>"
            b'<config id="c1"/>'
            b'<target id="t1"/>'
            b'<scanner id="s1"/>'
            b"<alterable>0</alterable>"
            b"</create_task>",
        )

    def test_create_audit_with_hosts_ordering(self):
        request = Audits.create_audit(
            name="foo",
            policy_id="c1",
            target_id="t1",
            scanner_id="s1",
            hosts_ordering=HostsOrdering.RANDOM,
        )

        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>foo</name>"
            b"<usage_type>audit</usage_type>"
            b'<config id="c1"/>'
            b'<target id="t1"/>'
            b'<scanner id="s1"/>'
            b"<hosts_ordering>random</hosts_ordering>"
            b"</create_task>",
        )

        request = Audits.create_audit(
            name="foo",
            policy_id="c1",
            target_id="t1",
            scanner_id="s1",
            hosts_ordering="random",
        )

        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>foo</name>"
            b"<usage_type>audit</usage_type>"
            b'<config id="c1"/>'
            b'<target id="t1"/>'
            b'<scanner id="s1"/>'
            b"<hosts_ordering>random</hosts_ordering>"
            b"</create_task>",
        )

    def test_create_audit_with_schedule_id(self):
        request = Audits.create_audit(
            name="foo",
            policy_id="c1",
            target_id="t1",
            scanner_id="s1",
            schedule_id="s1",
        )

        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>foo</name>"
            b"<usage_type>audit</usage_type>"
            b'<config id="c1"/>'
            b'<target id="t1"/>'
            b'<scanner id="s1"/>'
            b'<schedule id="s1"/>'
            b"</create_task>",
        )

    def test_create_audit_with_schedule_and_schedule_periods(self):
        request = Audits.create_audit(
            name="foo",
            policy_id="c1",
            target_id="t1",
            scanner_id="s1",
            schedule_id="s1",
            schedule_periods=0,
        )

        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>foo</name>"
            b"<usage_type>audit</usage_type>"
            b'<config id="c1"/>'
            b'<target id="t1"/>'
            b'<scanner id="s1"/>'
            b'<schedule id="s1"/>'
            b"<schedule_periods>0</schedule_periods>"
            b"</create_task>",
        )

        request = Audits.create_audit(
            name="foo",
            policy_id="c1",
            target_id="t1",
            scanner_id="s1",
            schedule_id="s1",
            schedule_periods=5,
        )

        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>foo</name>"
            b"<usage_type>audit</usage_type>"
            b'<config id="c1"/>'
            b'<target id="t1"/>'
            b'<scanner id="s1"/>'
            b'<schedule id="s1"/>'
            b"<schedule_periods>5</schedule_periods>"
            b"</create_task>",
        )

    def test_create_audit_with_observers(self):
        request = Audits.create_audit(
            name="foo",
            policy_id="c1",
            target_id="t1",
            scanner_id="s1",
            observers=["u1", "u2"],
        )

        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>foo</name>"
            b"<usage_type>audit</usage_type>"
            b'<config id="c1"/>'
            b'<target id="t1"/>'
            b'<scanner id="s1"/>'
            b"<observers>u1,u2</observers>"
            b"</create_task>",
        )

    def test_create_audit_with_preferences(self):
        request = Audits.create_audit(
            name="foo",
            policy_id="c1",
            target_id="t1",
            scanner_id="s1",
            preferences={"foo": "bar", "lorem": "ipsum"},
        )

        self.assertEqual(
            bytes(request),
            b"<create_task>"
            b"<name>foo</name>"
            b"<usage_type>audit</usage_type>"
            b'<config id="c1"/>'
            b'<target id="t1"/>'
            b'<scanner id="s1"/>'
            b"<preferences>"
            b"<preference>"
            b"<scanner_name>foo</scanner_name>"
            b"<value>bar</value>"
            b"</preference>"
            b"<preference>"
            b"<scanner_name>lorem</scanner_name>"
            b"<value>ipsum</value>"
            b"</preference>"
            b"</preferences>"
            b"</create_task>",
        )

    def test_create_audit_don_t_allow_container_task(self):
        with self.assertRaises(InvalidArgument):
            Audits.create_audit(
                name="foo",
                policy_id="c1",
                target_id="0",
                scanner_id="s1",
                observers="",
            )

        # target_id=0 is considered as False
        with self.assertRaises(RequiredArgument):
            Audits.create_audit(
                name="foo",
                policy_id="c1",
                target_id=0,
                scanner_id="s1",
                observers="",
            )

    def test_create_audit_missing_name(self):
        with self.assertRaises(RequiredArgument):
            Audits.create_audit(
                "", policy_id="c1", target_id="t1", scanner_id="s1"
            )

        with self.assertRaises(RequiredArgument):
            Audits.create_audit(
                None, policy_id="c1", target_id="t1", scanner_id="s1"
            )

    def test_create_audit_missing_policy_id(self):
        with self.assertRaises(RequiredArgument):
            Audits.create_audit(
                "foo", policy_id="", target_id="t1", scanner_id="s1"
            )

        with self.assertRaises(RequiredArgument):
            Audits.create_audit(
                "foo", policy_id=None, target_id="t1", scanner_id="s1"
            )

    def test_create_audit_missing_target_id(self):
        with self.assertRaises(RequiredArgument):
            Audits.create_audit(
                "foo", policy_id="c1", target_id="", scanner_id="s1"
            )

        with self.assertRaises(RequiredArgument):
            Audits.create_audit(
                "foo", policy_id="c1", target_id=None, scanner_id="s1"
            )

    def test_create_audit_missing_scanner_id(self):
        with self.assertRaises(RequiredArgument):
            Audits.create_audit(
                "foo", policy_id="c1", target_id="t1", scanner_id=""
            )

        with self.assertRaises(RequiredArgument):
            Audits.create_audit(
                "foo", policy_id="c1", target_id="t1", scanner_id=None
            )

    def test_create_audit_invalid_hosts_ordering(self):
        with self.assertRaises(InvalidArgument):
            Audits.create_audit(
                "foo",
                policy_id="c1",
                target_id="t1",
                scanner_id="s1",
                hosts_ordering="invalid",
            )

    def test_create_audit_with_schedule_and_invalid_schedule_periods(self):
        with self.assertRaises(InvalidArgument):
            Audits.create_audit(
                name="foo",
                policy_id="c1",
                target_id="t1",
                scanner_id="s1",
                schedule_id="s1",
                schedule_periods=-1,
            )

        with self.assertRaises(InvalidArgument):
            Audits.create_audit(
                name="foo",
                policy_id="c1",
                target_id="t1",
                scanner_id="s1",
                schedule_id="s1",
                schedule_periods="invalid",
            )

    def test_modify_audit(self):
        request = Audits.modify_audit("t1")

        self.assertEqual(bytes(request), b'<modify_task task_id="t1"/>')

    def test_modify_audit_missing_audit_id(self):
        with self.assertRaises(RequiredArgument):
            Audits.modify_audit(None)

        with self.assertRaises(RequiredArgument):
            Audits.modify_audit("")

    def test_modify_audit_with_name(self):
        request = Audits.modify_audit(audit_id="t1", name="foo")

        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="t1"><name>foo</name></modify_task>',
        )

    def test_modify_audit_with_policy_id(self):
        request = Audits.modify_audit(audit_id="t1", policy_id="c1")

        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="t1"><config id="c1"/></modify_task>',
        )

    def test_modify_audit_with_target_id(self):
        request = Audits.modify_audit(audit_id="t1", target_id="t1")

        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="t1"><target id="t1"/></modify_task>',
        )

    def test_modify_audit_with_scanner_id(self):
        request = Audits.modify_audit(audit_id="t1", scanner_id="s1")

        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="t1"><scanner id="s1"/></modify_task>',
        )

    def test_modify_audit_with_schedule_id(self):
        request = Audits.modify_audit(audit_id="t1", schedule_id="s1")

        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="t1"><schedule id="s1"/></modify_task>',
        )

    def test_modify_audit_with_comment(self):
        request = Audits.modify_audit(audit_id="t1", comment="bar")

        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="t1"><comment>bar</comment></modify_task>',
        )

    def test_modify_audit_with_alerts_ids(self):
        request = Audits.modify_audit(audit_id="t1", alert_ids=["a1", "a2"])

        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="t1">'
            b'<alert id="a1"/>'
            b'<alert id="a2"/>'
            b"</modify_task>",
        )

    def test_modify_audit_with_empty_alert_ids(self):
        request = Audits.modify_audit(audit_id="t1", alert_ids=[])

        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="t1"><alert id="0"/></modify_task>',
        )

    def test_modify_audit_with_alterable(self):
        request = Audits.modify_audit(audit_id="t1", alterable=True)

        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="t1"><alterable>1</alterable></modify_task>',
        )

        request = Audits.modify_audit(audit_id="t1", alterable=False)

        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="t1"><alterable>0</alterable></modify_task>',
        )

    def test_modify_audit_with_hosts_ordering(self):
        request = Audits.modify_audit(
            audit_id="t1", hosts_ordering=HostsOrdering.RANDOM
        )
        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="t1">'
            b"<hosts_ordering>random</hosts_ordering>"
            b"</modify_task>",
        )

        request = Audits.modify_audit(audit_id="t1", hosts_ordering="random")

        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="t1">'
            b"<hosts_ordering>random</hosts_ordering>"
            b"</modify_task>",
        )

    def test_modify_audit_with_schedule_periods(self):
        request = Audits.modify_audit(audit_id="t1", schedule_periods=0)

        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="t1">'
            b"<schedule_periods>0</schedule_periods>"
            b"</modify_task>",
        )

        request = Audits.modify_audit(audit_id="t1", schedule_periods=5)

        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="t1">'
            b"<schedule_periods>5</schedule_periods>"
            b"</modify_task>",
        )

    def test_modify_audit_with_observers(self):
        request = Audits.modify_audit(audit_id="t1", observers=["u1", "u2"])

        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="t1">'
            b"<observers>u1,u2</observers>"
            b"</modify_task>",
        )

    def test_modify_audit_with_preferences(self):
        request = Audits.modify_audit(
            audit_id="t1", preferences={"foo": "bar", "lorem": "ipsum"}
        )

        self.assertEqual(
            bytes(request),
            b'<modify_task task_id="t1">'
            b"<preferences>"
            b"<preference>"
            b"<scanner_name>foo</scanner_name>"
            b"<value>bar</value>"
            b"</preference>"
            b"<preference>"
            b"<scanner_name>lorem</scanner_name>"
            b"<value>ipsum</value>"
            b"</preference>"
            b"</preferences>"
            b"</modify_task>",
        )

    def test_modify_audit_invalid_hosts_ordering(self):
        with self.assertRaises(InvalidArgument):
            Audits.modify_audit(audit_id="t1", hosts_ordering="foo")

    def test_modify_audit_invalid_schedule_periods(self):
        with self.assertRaises(InvalidArgument):
            Audits.modify_audit(audit_id="t1", schedule_periods="foo")

        with self.assertRaises(InvalidArgument):
            Audits.modify_audit(audit_id="t1", schedule_periods=-1)

    def test_clone_audit(self):
        request = Audits.clone_audit("t1")

        self.assertEqual(
            bytes(request),
            b"<create_task><copy>t1</copy></create_task>",
        )

    def test_clone_audit_missing_audit_id(self):
        with self.assertRaises(RequiredArgument):
            Audits.clone_audit(None)

        with self.assertRaises(RequiredArgument):
            Audits.clone_audit("")

    def test_delete_audit(self):
        request = Audits.delete_audit("t1")

        self.assertEqual(
            bytes(request),
            b'<delete_task task_id="t1" ultimate="0"/>',
        )

    def test_delete_audit_with_ultimate(self):
        request = Audits.delete_audit("t1", ultimate=True)

        self.assertEqual(
            bytes(request),
            b'<delete_task task_id="t1" ultimate="1"/>',
        )

        request = Audits.delete_audit("t1", ultimate=False)

        self.assertEqual(
            bytes(request),
            b'<delete_task task_id="t1" ultimate="0"/>',
        )

    def test_delete_audit_missing_audit_id(self):
        with self.assertRaises(RequiredArgument):
            Audits.delete_audit(None)

        with self.assertRaises(RequiredArgument):
            Audits.delete_audit("")

    def test_get_audits(self):
        request = Audits.get_audits()

        self.assertEqual(bytes(request), b'<get_tasks usage_type="audit"/>')

    def test_get_audits_with_filter_string(self):
        request = Audits.get_audits(filter_string="name=foo")

        self.assertEqual(
            bytes(request),
            b'<get_tasks usage_type="audit" filter="name=foo"/>',
        )

    def test_get_audits_with_filter_id(self):
        request = Audits.get_audits(filter_id="f1")

        self.assertEqual(
            bytes(request),
            b'<get_tasks usage_type="audit" filt_id="f1"/>',
        )

    def test_get_audits_with_trash(self):
        request = Audits.get_audits(trash=True)

        self.assertEqual(
            bytes(request),
            b'<get_tasks usage_type="audit" trash="1"/>',
        )

        request = Audits.get_audits(trash=False)

        self.assertEqual(
            bytes(request),
            b'<get_tasks usage_type="audit" trash="0"/>',
        )

    def test_get_audits_with_details(self):
        request = Audits.get_audits(details=True)

        self.assertEqual(
            bytes(request),
            b'<get_tasks usage_type="audit" details="1"/>',
        )

        request = Audits.get_audits(details=False)

        self.assertEqual(
            bytes(request),
            b'<get_tasks usage_type="audit" details="0"/>',
        )

    def test_get_audits_with_schedules_only(self):
        request = Audits.get_audits(schedules_only=True)

        self.assertEqual(
            bytes(request),
            b'<get_tasks usage_type="audit" schedules_only="1"/>',
        )

        request = Audits.get_audits(schedules_only=False)

        self.assertEqual(
            bytes(request),
            b'<get_tasks usage_type="audit" schedules_only="0"/>',
        )

    def test_get_audit(self):
        request = Audits.get_audit("t1")

        self.assertEqual(
            bytes(request),
            b'<get_tasks task_id="t1" usage_type="audit" details="1"/>',
        )

    def test_get_audit_missing_audit_id(self):
        with self.assertRaises(RequiredArgument):
            Audits.get_audit(None)

        with self.assertRaises(RequiredArgument):
            Audits.get_audit("")

    def test_start_audit(self):
        request = Audits.start_audit("t1")

        self.assertEqual(
            bytes(request),
            b'<start_task task_id="t1"/>',
        )

    def test_start_audit_missing_audit_id(self):
        with self.assertRaises(RequiredArgument):
            Audits.start_audit(None)

        with self.assertRaises(RequiredArgument):
            Audits.start_audit("")

    def test_stop_audit(self):
        request = Audits.stop_audit("t1")

        self.assertEqual(
            bytes(request),
            b'<stop_task task_id="t1"/>',
        )

    def test_stop_audit_missing_audit_id(self):
        with self.assertRaises(RequiredArgument):
            Audits.stop_audit(None)

        with self.assertRaises(RequiredArgument):
            Audits.stop_audit("")

    def test_resume_audit(self):
        request = Audits.resume_audit("t1")

        self.assertEqual(
            bytes(request),
            b'<resume_task task_id="t1"/>',
        )

    def test_resume_audit_missing_audit_id(self):
        with self.assertRaises(RequiredArgument):
            Audits.resume_audit(None)

        with self.assertRaises(RequiredArgument):
            Audits.resume_audit("")



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetSchedulesTestMixin:
    def test_get_schedules(self):
        self.gmp.get_schedules()

        self.connection.send.has_been_called_with(b"<get_schedules/>")

    def test_get_schedules_with_filter_string(self):
        self.gmp.get_schedules(filter_string="foo=bar")

        self.connection.send.has_been_called_with(
            b'<get_schedules filter="foo=bar"/>'
        )

    def test_get_schedules_with_filter_id(self):
        self.gmp.get_schedules(filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_schedules filt_id="f1"/>'
        )

    def test_get_schedules_with_trash(self):
        self.gmp.get_schedules(trash=True)

        self.connection.send.has_been_called_with(b'<get_schedules trash="1"/>')

        self.gmp.get_schedules(trash=False)

        self.connection.send.has_been_called_with(b'<get_schedules trash="0"/>')

    def test_get_schedules_with_tasks(self):
        self.gmp.get_schedules(tasks=True)

        self.connection.send.has_been_called_with(b'<get_schedules tasks="1"/>')

        self.gmp.get_schedules(tasks=False)

        self.connection.send.has_been_called_with(b'<get_schedules tasks="0"/>')



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpGetCpeTestMixin:
    def test_get_cpe(self):
        self.gmp.get_cpe(cpe_id="i1")

        self.connection.send.has_been_called_with(
            b'<get_info info_id="i1" type="CPE" details="1"/>'
        )

        self.gmp.get_cpe("i1")

        self.connection.send.has_been_called_with(
            b'<get_info info_id="i1" type="CPE" details="1"/>'
        )

    def test_get_cpe_missing_cpe_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_cpe(cpe_id="")

        with self.assertRaises(RequiredArgument):
            self.gmp.get_cpe("")

        with self.assertRaises(RequiredArgument):
            self.gmp.get_cpe(cpe_id=None)



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import check_port, to_bool, to_comma_list
from gvm.xml import XmlCommand

from .._entity_id import EntityID
from ._severity import Severity


class Overrides:
    @classmethod
    def create_override(
        cls,
        text: str,
        nvt_oid: str,
        *,
        days_active: int | None = None,
        hosts: list[str] | None = None,
        port: str | None = None,
        result_id: EntityID | None = None,
        severity: Severity | None = None,
        new_severity: Severity | None = None,
        task_id: EntityID | None = None,
    ) -> Request:
        """Create a new override

        Args:
            text: Text of the new override
            nvt_oid: OID of the nvt to which override applies
            days_active: Days override will be active. -1 on always, 0 off
            hosts: A list of host addresses
            port: Port to which the override applies, needs to be a string
                  in the form {number}/{protocol}
            result_id: UUID of a result to which override applies
            severity: Severity to which override applies
            new_severity: New severity for result
            task_id: UUID of task to which override applies
        """
        if not text:
            raise RequiredArgument(
                function=cls.create_override.__name__, argument="text"
            )

        if not nvt_oid:
            raise RequiredArgument(
                function=cls.create_override.__name__, argument="nvt_oid"
            )

        cmd = XmlCommand("create_override")
        cmd.add_element("text", text)
        cmd.add_element("nvt", attrs={"oid": nvt_oid})

        if days_active is not None:
            cmd.add_element("active", str(days_active))

        if hosts:
            cmd.add_element("hosts", to_comma_list(hosts))

        if port:
            if check_port(port):
                cmd.add_element("port", port)
            else:
                raise InvalidArgument(
                    function=cls.create_override.__name__, argument="port"
                )

        if result_id:
            cmd.add_element("result", attrs={"id": result_id})

        if severity is not None:
            cmd.add_element("severity", str(severity))

        if new_severity is not None:
            cmd.add_element("new_severity", str(new_severity))

        if task_id:
            cmd.add_element("task", attrs={"id": task_id})

        return cmd

    @classmethod
    def modify_override(
        cls,
        override_id: EntityID,
        text: str,
        *,
        days_active: int | None = None,
        hosts: list[str] | None = None,
        port: str | None = None,
        result_id: EntityID | None = None,
        severity: Severity | None = None,
        new_severity: Severity | None = None,
        task_id: EntityID | None = None,
    ) -> Request:
        """Modify an existing override.

        Args:
            override_id: UUID of override to modify.
            text: The text of the override.
            days_active: Days override will be active. -1 on always,
                0 off.
            hosts: A list of host addresses
            port: Port to which the override applies, needs to be a string
                  in the form {number}/{protocol}
            result_id: Result to which override applies.
            severity: Severity to which override applies.
            new_severity: New severity score for result.
            task_id: Task to which override applies.
        """
        if not override_id:
            raise RequiredArgument(
                function=cls.modify_override.__name__, argument="override_id"
            )

        if not text:
            raise RequiredArgument(
                function=cls.modify_override.__name__, argument="text"
            )

        cmd = XmlCommand("modify_override")
        cmd.set_attribute("override_id", str(override_id))
        cmd.add_element("text", text)

        if days_active is not None:
            cmd.add_element("active", str(days_active))

        if hosts:
            cmd.add_element("hosts", to_comma_list(hosts))

        if port:
            if check_port(port):
                cmd.add_element("port", str(port))
            else:
                raise InvalidArgument(
                    function=cls.modify_override.__name__, argument="port"
                )

        if result_id:
            cmd.add_element("result", attrs={"id": str(result_id)})

        if severity is not None:
            cmd.add_element("severity", str(severity))

        if new_severity is not None:
            cmd.add_element("new_severity", str(new_severity))

        if task_id:
            cmd.add_element("task", attrs={"id": str(task_id)})

        return cmd

    @classmethod
    def clone_override(cls, override_id: EntityID) -> Request:
        """Clone an existing override

        Args:
            override_id: UUID of an existing override to clone from
        """
        if not override_id:
            raise RequiredArgument(
                function=cls.clone_override.__name__, argument="override_id"
            )

        cmd = XmlCommand("create_override")
        cmd.add_element("copy", str(override_id))
        return cmd

    @classmethod
    def delete_override(
        cls, override_id: EntityID, *, ultimate: bool | None = False
    ) -> Request:
        """Delete an existing override

        Args:
            override_id: UUID of an existing override to delete
            ultimate: Whether to remove entirely, or to the trashcan.
        """
        if not override_id:
            raise RequiredArgument(
                function=cls.delete_override.__name__, argument="override_id"
            )

        cmd = XmlCommand("delete_override")
        cmd.set_attribute("override_id", str(override_id))
        cmd.set_attribute("ultimate", to_bool(ultimate))
        return cmd

    @staticmethod
    def get_overrides(
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        details: bool | None = None,
        result: bool | None = None,
    ) -> Request:
        """Request a list of overrides

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            details: Whether to include full details
            result: Whether to include results using the override
        """
        cmd = XmlCommand("get_overrides")
        cmd.add_filter(filter_string, filter_id)

        if details is not None:
            cmd.set_attribute("details", to_bool(details))

        if result is not None:
            cmd.set_attribute("result", to_bool(result))

        return cmd

    @classmethod
    def get_override(cls, override_id: EntityID) -> Request:
        """Request a single override

        Args:
            override_id: UUID of an existing override
        """
        cmd = XmlCommand("get_overrides")

        if not override_id:
            raise RequiredArgument(
                function=cls.get_override.__name__, argument="override_id"
            )

        cmd.set_attribute("override_id", str(override_id))

        # for single entity always request all details
        cmd.set_attribute("details", "1")
        return cmd



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpGetAlertTestMixin:
    def test_get_alert(self):
        self.gmp.get_alert("a1")

        self.connection.send.has_been_called_with(
            b'<get_alerts alert_id="a1"/>'
        )

        self.gmp.get_alert(alert_id="a1")

        self.connection.send.has_been_called_with(
            b'<get_alerts alert_id="a1"/>'
        )

    def test_get_alert_invalid_alert_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_alert(alert_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_alert(alert_id="")

    def test_get_alert_with_tasks(self):
        self.gmp.get_alert(alert_id="a1", tasks=True)

        self.connection.send.has_been_called_with(
            b'<get_alerts alert_id="a1" tasks="1"/>'
        )

        self.gmp.get_alert(alert_id="a1", tasks=False)

        self.connection.send.has_been_called_with(
            b'<get_alerts alert_id="a1" tasks="0"/>'
        )



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpCreateRoleTestMixin:
    def test_create_role(self):
        self.gmp.create_role(name="foo")

        self.connection.send.has_been_called_with(
            b"<create_role><name>foo</name></create_role>"
        )

    def test_missing_name(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_role(None)

        with self.assertRaises(RequiredArgument):
            self.gmp.create_role("")

    def test_create_role_with_comment(self):
        self.gmp.create_role(name="foo", comment="bar")

        self.connection.send.has_been_called_with(
            b"<create_role><name>foo</name><comment>bar</comment></create_role>"
        )

    def test_create_role_with_users(self):
        self.gmp.create_role(name="foo", users=[])

        self.connection.send.has_been_called_with(
            b"<create_role><name>foo</name></create_role>"
        )

        self.gmp.create_role(name="foo", users=["u1", "u2"])

        self.connection.send.has_been_called_with(
            b"<create_role><name>foo</name><users>u1,u2</users></create_role>"
        )



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


from gvm.errors import RequiredArgument


class GMPGetReportConfigTestMixin:
    def test_get_report_config(self):
        self.gmp.get_report_config("report_config_id")

        self.connection.send.has_been_called_with(
            b'<get_report_configs report_config_id="report_config_id" details="1"/>',
        )

    def test_get_report_config_missing_report_config_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_report_config(None)
        with self.assertRaises(RequiredArgument):
            self.gmp.get_report_config("")



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


from gvm._enum import Enum
from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_bool
from gvm.xml import XmlCommand, XmlError

from .._entity_id import EntityID


class ReportFormatType(Enum):
    """Enum for builtin report formats"""

    ANONYMOUS_XML = "5057e5cc-b825-11e4-9d0e-28d24461215b"
    ARF = "910200ca-dc05-11e1-954f-406186ea4fc5"
    CPE = "5ceff8ba-1f62-11e1-ab9f-406186ea4fc5"
    CSV_HOSTS = '9087b18c-626c-11e3-8892-406186ea4fc5"'
    CSV_RESULTS = "c1645568-627a-11e3-a660-406186ea4fc5"
    GCR_PDF = "dc51a40a-c022-11e9-b02d-3f7ca5bdcb11"
    GSR_HTML = "ffa123c9-a2d2-409e-bbbb-a6c1385dbeaa"
    GSR_PDF = "35ba7077-dc85-42ef-87c9-b0eda7e903b6"
    GXCR_PDF = "f0d348de-c022-11e9-bc4c-4bf1d5e1a8ca"
    GXR_PDF = "ebbc7f34-8ae5-11e1-b07b-001f29eadec8"
    ITG = "77bd6c4a-1f62-11e1-abf0-406186ea4fc5"
    LATEX = "a684c02c-b531-11e1-bdc2-406186ea4fc5"
    NBE = "9ca6fe72-1f62-11e1-9e7c-406186ea4fc5"
    PDF = "c402cc3e-b531-11e1-9163-406186ea4fc5"
    SVG = "9e5e5deb-879e-4ecc-8be6-a71cd0875cdd"
    TXT = "a3810a62-1f62-11e1-9219-406186ea4fc5"
    VERINICE_ISM = "c15ad349-bd8d-457a-880a-c7056532ee15"
    VERINICE_ITG = "50c9950a-f326-11e4-800c-28d24461215b"
    XML = "a994b278-1f62-11e1-96ac-406186ea4fc5"

    def __str__(self) -> str:
        return self.value


class ReportFormats:
    @classmethod
    def clone_report_format(
        cls, report_format_id: EntityID | ReportFormatType
    ) -> Request:
        """Clone a report format from an existing one

        Args:
            report_format_id: UUID of the existing report format
                              or ReportFormatType (enum)
        """
        if not report_format_id:
            raise RequiredArgument(
                function=cls.clone_report_format.__name__,
                argument="report_format_id",
            )

        cmd = XmlCommand("create_report_format")
        cmd.add_element("copy", str(report_format_id))
        return cmd

    @classmethod
    def delete_report_format(
        cls,
        report_format_id: EntityID | ReportFormatType,
        *,
        ultimate: bool | None = False,
    ) -> Request:
        """Deletes an existing report format

        Args:
            report_format_id: UUID of the report format to be deleted.
                              or ReportFormatType (enum)
            ultimate: Whether to remove entirely, or to the trashcan.
        """
        if not report_format_id:
            raise RequiredArgument(
                function=cls.delete_report_format.__name__,
                argument="report_format_id",
            )

        cmd = XmlCommand("delete_report_format")

        cmd.set_attribute("report_format_id", str(report_format_id))

        cmd.set_attribute("ultimate", to_bool(ultimate))

        return cmd

    @staticmethod
    def get_report_formats(
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        trash: bool | None = None,
        alerts: bool | None = None,
        params: bool | None = None,
        details: bool | None = None,
    ) -> Request:
        """Request a list of report formats

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            trash: Whether to get the trashcan report formats instead
            alerts: Whether to include alerts that use the report format
            params: Whether to include report format parameters
            details: Include report format file, signature and parameters
        """
        cmd = XmlCommand("get_report_formats")

        cmd.add_filter(filter_string, filter_id)

        if details is not None:
            cmd.set_attribute("details", to_bool(details))

        if alerts is not None:
            cmd.set_attribute("alerts", to_bool(alerts))

        if params is not None:
            cmd.set_attribute("params", to_bool(params))

        if trash is not None:
            cmd.set_attribute("trash", to_bool(trash))

        return cmd

    @classmethod
    def get_report_format(
        cls, report_format_id: EntityID | ReportFormatType
    ) -> Request:
        """Request a single report format

        Args:
            report_format_id: UUID of an existing report format
                              or ReportFormatType (enum)
        """
        if not report_format_id:
            raise RequiredArgument(
                function=cls.get_report_format.__name__,
                argument="report_format_id",
            )

        cmd = XmlCommand("get_report_formats")

        cmd.set_attribute("report_format_id", str(report_format_id))

        # for single entity always request all details
        cmd.set_attribute("details", "1")
        return cmd

    @classmethod
    def import_report_format(cls, report_format: str) -> Request:
        """Import a report format from XML

        Args:
            report_format: Report format XML as string to import. This XML must
                contain a :code:`<get_report_formats_response>` root element.
        """
        if not report_format:
            raise RequiredArgument(
                function=cls.import_report_format.__name__,
                argument="report_format",
            )

        cmd = XmlCommand("create_report_format")

        try:
            cmd.append_xml_str(report_format)
        except XmlError as e:
            raise InvalidArgument(
                function=cls.import_report_format.__name__,
                argument="report_format",
            ) from e

        return cmd

    @classmethod
    def modify_report_format(
        cls,
        report_format_id: EntityID | ReportFormatType,
        *,
        active: bool | None = None,
        name: str | None = None,
        summary: str | None = None,
        param_name: str | None = None,
        param_value: str | None = None,
    ) -> Request:
        """Modifies an existing report format.

        Args:
            report_format_id: UUID of report format to modify
                              or ReportFormatType (enum)
            active: Whether the report format is active.
            name: The name of the report format.
            summary: A summary of the report format.
            param_name: The name of the param.
            param_value: The value of the param.
        """
        if not report_format_id:
            raise RequiredArgument(
                function=cls.modify_report_format.__name__,
                argument="report_format_id ",
            )

        cmd = XmlCommand("modify_report_format")

        cmd.set_attribute("report_format_id", str(report_format_id))

        if active is not None:
            cmd.add_element("active", to_bool(active))

        if name:
            cmd.add_element("name", name)

        if summary:
            cmd.add_element("summary", summary)

        if param_name:
            xml_param = cmd.add_element("param")
            xml_param.add_element("name", param_name)

            if param_value is not None:
                xml_param.add_element("value", param_value)

        return cmd

    @classmethod
    def verify_report_format(
        cls, report_format_id: EntityID | ReportFormatType
    ) -> Request:
        """Verify an existing report format

        Verifies the trust level of an existing report format. It will be
        checked whether the signature of the report format currently matches the
        report format. This includes the script and files used to generate
        reports of this format. It is *not* verified if the report format works
        as expected by the user.

        Args:
            report_format_id: UUID of the report format to be verified
                              or ReportFormatType (enum)
        """
        if not report_format_id:
            raise RequiredArgument(
                function=cls.verify_report_format.__name__,
                argument="report_format_id",
            )

        cmd = XmlCommand("verify_report_format")
        cmd.set_attribute("report_format_id", str(report_format_id))
        return cmd



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetOverridesTestMixin:
    def test_get_overrides(self):
        self.gmp.get_overrides()

        self.connection.send.has_been_called_with(b"<get_overrides/>")

    def test_get_overrides_with_filter_string(self):
        self.gmp.get_overrides(filter_string="foo=bar")

        self.connection.send.has_been_called_with(
            b'<get_overrides filter="foo=bar"/>'
        )

    def test_get_overrides_with_filter_id(self):
        self.gmp.get_overrides(filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_overrides filt_id="f1"/>'
        )

    def test_get_overrides_with_details(self):
        self.gmp.get_overrides(details=True)

        self.connection.send.has_been_called_with(
            b'<get_overrides details="1"/>'
        )

        self.gmp.get_overrides(details=False)

        self.connection.send.has_been_called_with(
            b'<get_overrides details="0"/>'
        )

    def test_get_overrides_with_result(self):
        self.gmp.get_overrides(result=True)

        self.connection.send.has_been_called_with(
            b'<get_overrides result="1"/>'
        )

        self.gmp.get_overrides(result=False)

        self.connection.send.has_been_called_with(
            b'<get_overrides result="0"/>'
        )



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from numbers import Integral

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_bool, to_comma_list
from gvm.xml import XmlCommand

from .._entity_id import EntityID
from ._hosts import HostsOrdering


class Audits:
    @classmethod
    def create_audit(
        cls,
        name: str,
        policy_id: EntityID,
        target_id: EntityID,
        scanner_id: EntityID,
        *,
        alterable: bool | None = None,
        hosts_ordering: HostsOrdering | str | None = None,
        schedule_id: str | None = None,
        alert_ids: list[EntityID] | None = None,
        comment: str | None = None,
        schedule_periods: int | None = None,
        observers: list[EntityID] | None = None,
        preferences: dict[str, str] | None = None,
    ) -> Request:
        """Create a new audit

        Args:
            name: Name of the new audit
            policy_id: UUID of policy to use by the audit
            target_id: UUID of target to be scanned
            scanner_id: UUID of scanner to use for scanning the target
            comment: Comment for the audit
            alterable: Whether the task should be alterable
            alert_ids: List of UUIDs for alerts to be applied to the audit
            hosts_ordering: The order hosts are scanned in
            schedule_id: UUID of a schedule when the audit should be run.
            schedule_periods: A limit to the number of times the audit will be
                scheduled, or 0 for no limit
            observers: List of names or ids of users which should be allowed to
                observe this audit
            preferences: Name/Value pairs of scanner preferences.
        """

        if not name:
            raise RequiredArgument(
                function=cls.create_audit.__name__, argument="name"
            )
        if not policy_id:
            raise RequiredArgument(
                function=cls.create_audit.__name__, argument="policy_id"
            )

        if not target_id:
            raise RequiredArgument(
                function=cls.create_audit.__name__, argument="target_id"
            )

        if not scanner_id:
            raise RequiredArgument(
                function=cls.create_audit.__name__, argument="scanner_id"
            )

        # don't allow to create a container task with create_task
        if target_id == "0":
            raise InvalidArgument(
                function=cls.create_audit.__name__, argument="target_id"
            )

        cmd = XmlCommand("create_task")
        cmd.add_element("name", name)
        cmd.add_element("usage_type", "audit")
        cmd.add_element("config", attrs={"id": str(policy_id)})
        cmd.add_element("target", attrs={"id": str(target_id)})
        cmd.add_element("scanner", attrs={"id": str(scanner_id)})

        if comment:
            cmd.add_element("comment", comment)

        if alterable is not None:
            cmd.add_element("alterable", to_bool(alterable))

        if hosts_ordering:
            if not isinstance(hosts_ordering, HostsOrdering):
                hosts_ordering = HostsOrdering(hosts_ordering)
            cmd.add_element("hosts_ordering", hosts_ordering.value)

        if alert_ids:
            for alert in alert_ids:
                cmd.add_element("alert", attrs={"id": str(alert)})

        if schedule_id:
            cmd.add_element("schedule", attrs={"id": str(schedule_id)})

            if schedule_periods is not None:
                if (
                    not isinstance(schedule_periods, Integral)
                    or schedule_periods < 0
                ):
                    raise InvalidArgument(
                        "schedule_periods must be an integer greater or equal "
                        "than 0"
                    )
                cmd.add_element("schedule_periods", str(schedule_periods))

        if observers:
            # gvmd splits by comma and space
            # gvmd tries to lookup each value as user name and afterwards as
            # user id. So both user name and user id are possible
            cmd.add_element("observers", to_comma_list(observers))

        if preferences is not None:
            xml_prefs = cmd.add_element("preferences")
            for pref_name, pref_value in preferences.items():
                xml_pref = xml_prefs.add_element("preference")
                xml_pref.add_element("scanner_name", pref_name)
                xml_pref.add_element("value", str(pref_value))

        return cmd

    @classmethod
    def modify_audit(
        cls,
        audit_id: EntityID,
        *,
        name: str | None = None,
        policy_id: EntityID | None = None,
        target_id: EntityID | None = None,
        scanner_id: EntityID | None = None,
        alterable: bool | None = None,
        hosts_ordering: str | HostsOrdering | None = None,
        schedule_id: EntityID | None = None,
        schedule_periods: int | None = None,
        comment: str | None = None,
        alert_ids: list[EntityID] | None = None,
        observers: list[EntityID] | None = None,
        preferences: dict[str, str] | None = None,
    ) -> Request:
        """Modifies an existing audit.

        Args:
            audit_id: UUID of audit to modify.
            name: The name of the audit.
            policy_id: UUID of policy to use by the audit
            target_id: UUID of target to be scanned
            scanner_id: UUID of scanner to use for scanning the target
            comment: The comment on the audit.
            alert_ids: List of UUIDs for alerts to be applied to the audit
            hosts_ordering: The order hosts are scanned in
            schedule_id: UUID of a schedule when the audit should be run.
            schedule_periods: A limit to the number of times the audit will be
                scheduled, or 0 for no limit.
            observers: List of names or ids of users which should be allowed to
                observe this audit
            preferences: Name/Value pairs of scanner preferences.
        """
        if not audit_id:
            raise RequiredArgument(
                function=cls.modify_audit.__name__, argument="task_id argument"
            )

        cmd = XmlCommand("modify_task")
        cmd.set_attribute("task_id", str(audit_id))

        if name:
            cmd.add_element("name", name)

        if comment:
            cmd.add_element("comment", comment)

        if policy_id:
            cmd.add_element("config", attrs={"id": str(policy_id)})

        if target_id:
            cmd.add_element("target", attrs={"id": str(target_id)})

        if alterable is not None:
            cmd.add_element("alterable", to_bool(alterable))

        if hosts_ordering:
            if not isinstance(hosts_ordering, HostsOrdering):
                hosts_ordering = HostsOrdering(hosts_ordering)
            cmd.add_element("hosts_ordering", hosts_ordering.value)

        if scanner_id:
            cmd.add_element("scanner", attrs={"id": str(scanner_id)})

        if schedule_id:
            cmd.add_element("schedule", attrs={"id": str(schedule_id)})

        if schedule_periods is not None:
            if (
                not isinstance(schedule_periods, Integral)
                or schedule_periods < 0
            ):
                raise InvalidArgument(
                    "schedule_periods must be an integer greater or equal "
                    "than 0"
                )
            cmd.add_element("schedule_periods", str(schedule_periods))

        if alert_ids is not None:
            if len(alert_ids) == 0:
                cmd.add_element("alert", attrs={"id": "0"})
            else:
                for alert in alert_ids:
                    cmd.add_element("alert", attrs={"id": str(alert)})

        if observers is not None:
            cmd.add_element("observers", to_comma_list(observers))

        if preferences is not None:
            xml_prefs = cmd.add_element("preferences")
            for pref_name, pref_value in preferences.items():
                xml_pref = xml_prefs.add_element("preference")
                xml_pref.add_element("scanner_name", pref_name)
                xml_pref.add_element("value", str(pref_value))

        return cmd

    @classmethod
    def clone_audit(cls, audit_id: EntityID) -> Request:
        """Clone an existing audit

        Args:
            audit_id: UUID of the audit to clone
        """
        if not audit_id:
            raise RequiredArgument(
                function=cls.clone_audit.__name__, argument="task_id"
            )

        cmd = XmlCommand("create_task")
        cmd.add_element("copy", str(audit_id))
        return cmd

    @classmethod
    def delete_audit(
        cls, audit_id: EntityID, *, ultimate: bool | None = False
    ) -> Request:
        """Delete an existing audit

        Args:
            audit_id: UUID of the audit to be deleted.
            ultimate: Whether to remove entirely, or to the trashcan.
        """
        if not audit_id:
            raise RequiredArgument(
                function=cls.delete_audit.__name__, argument="task_id"
            )

        cmd = XmlCommand("delete_task")
        cmd.set_attribute("task_id", str(audit_id))
        cmd.set_attribute("ultimate", to_bool(ultimate))
        return cmd

    @staticmethod
    def get_audits(
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        trash: bool | None = None,
        details: bool | None = None,
        schedules_only: bool | None = None,
    ) -> Request:
        """Request a list of audits

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            trash: Whether to get the trashcan audits instead
            details: Whether to include full audit details
            schedules_only: Whether to only include id, name and schedule
                details
        """
        cmd = XmlCommand("get_tasks")
        cmd.set_attribute("usage_type", "audit")
        cmd.add_filter(filter_string, filter_id)

        if trash is not None:
            cmd.set_attribute("trash", to_bool(trash))

        if details is not None:
            cmd.set_attribute("details", to_bool(details))

        if schedules_only is not None:
            cmd.set_attribute("schedules_only", to_bool(schedules_only))

        return cmd

    @classmethod
    def get_audit(cls, audit_id: EntityID) -> Request:
        """Request a single audit

        Args:
            audit_id: UUID of an existing audit
        """
        if not audit_id:
            raise RequiredArgument(
                function=cls.get_audit.__name__, argument="audit_id"
            )

        cmd = XmlCommand("get_tasks")
        cmd.set_attribute("task_id", str(audit_id))
        cmd.set_attribute("usage_type", "audit")

        # for single entity always request all details
        cmd.set_attribute("details", "1")
        return cmd

    @classmethod
    def resume_audit(cls, audit_id: EntityID) -> Request:
        """Resume an existing stopped audit

        Args:
            audit_id: UUID of the audit to be resumed
        """
        if not audit_id:
            raise RequiredArgument(
                function=cls.resume_audit.__name__, argument="audit_id"
            )

        cmd = XmlCommand("resume_task")
        cmd.set_attribute("task_id", str(audit_id))
        return cmd

    @classmethod
    def start_audit(cls, audit_id: EntityID) -> Request:
        """Start an existing audit

        Args:
            audit_id: UUID of the audit to be started
        """
        if not audit_id:
            raise RequiredArgument(
                function=cls.start_audit.__name__, argument="audit_id"
            )

        cmd = XmlCommand("start_task")
        cmd.set_attribute("task_id", str(audit_id))
        return cmd

    @classmethod
    def stop_audit(cls, audit_id: EntityID) -> Request:
        """Stop an existing running audit

        Args:
            audit_id: UUID of the audit to be stopped
        """
        if not audit_id:
            raise RequiredArgument(
                function=cls.stop_audit.__name__, argument="audit_id"
            )

        cmd = XmlCommand("stop_task")
        cmd.set_attribute("task_id", str(audit_id))
        return cmd



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm._enum import Enum
from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_bool, to_comma_list
from gvm.xml import XmlCommand

from .._entity_id import EntityID


class UserAuthType(Enum):
    """Enum for Sources allowed for authentication for the user"""

    FILE = "file"
    LDAP_CONNECT = "ldap_connect"
    RADIUS_CONNECT = "radius_connect"


class Users:
    @classmethod
    def create_user(
        cls,
        name: str,
        *,
        password: str | None = None,
        hosts: list[str] | None = None,
        hosts_allow: bool | None = False,
        role_ids: list[EntityID] | None = None,
    ) -> Request:
        """Create a new user

        Args:
            name: Name of the user
            password: Password of the user
            hosts: A list of host addresses (IPs, DNS names)
            hosts_allow: If True allow only access to passed hosts otherwise
                deny access. Default is False for deny hosts.
            role_ids: A list of role UUIDs for the user
        """
        if not name:
            raise RequiredArgument(
                function=cls.create_user.__name__, argument="name"
            )

        cmd = XmlCommand("create_user")
        cmd.add_element("name", name)

        if password:
            cmd.add_element("password", password)

        if hosts:
            cmd.add_element(
                "hosts",
                to_comma_list(hosts),
                attrs={"allow": to_bool(hosts_allow)},
            )

        if role_ids:
            for role in role_ids:
                cmd.add_element("role", attrs={"id": str(role)})

        return cmd

    @classmethod
    def modify_user(
        cls,
        user_id: EntityID,
        *,
        name: str | None = None,
        comment: str | None = None,
        password: str | None = None,
        auth_source: UserAuthType | str | None = None,
        role_ids: list[EntityID] | None = None,
        hosts: list[str] | None = None,
        hosts_allow: bool | None = False,
        group_ids: list[EntityID] | None = None,
    ) -> Request:
        """Modify an existing user.

        Most of the fields need to be supplied
        for changing a single field even if no change is wanted for those.
        Else empty values are inserted for the missing fields instead.

        Args:
            user_id: UUID of the user to be modified.
            name: The new name for the user.
            comment: Comment on the user.
            password: The password for the user.
            auth_source: Source allowed for authentication for this user.
            roles_id: List of roles UUIDs for the user.
            hosts: User access rules: List of hosts.
            hosts_allow: Defines how the hosts list is to be interpreted.
                If False (default) the list is treated as a deny list.
                All hosts are allowed by default except those provided by
                the hosts parameter. If True the list is treated as a
                allow list. All hosts are denied by default except those
                provided by the hosts parameter.
            group_ids: List of group UUIDs for the user.
        """
        if not user_id:
            raise RequiredArgument(
                function=cls.modify_user.__name__, argument="user_id"
            )

        cmd = XmlCommand("modify_user")

        cmd.set_attribute("user_id", str(user_id))

        if name:
            cmd.add_element("new_name", name)

        if role_ids:
            for role in role_ids:
                cmd.add_element("role", attrs={"id": role})

        if hosts:
            cmd.add_element(
                "hosts",
                to_comma_list(hosts),
                attrs={"allow": to_bool(hosts_allow)},
            )

        if comment:
            cmd.add_element("comment", comment)

        if password:
            cmd.add_element("password", password)

        if auth_source:
            xml_auth_src = cmd.add_element("sources")
            if not isinstance(auth_source, UserAuthType):
                auth_source = UserAuthType(auth_source)
            xml_auth_src.add_element("source", auth_source.value)

        if group_ids:
            xml_groups = cmd.add_element("groups")
            for group_id in group_ids:
                xml_groups.add_element("group", attrs={"id": group_id})

        return cmd

    @classmethod
    def clone_user(cls, user_id: EntityID) -> Request:
        """Clone an existing user.

        Args:
            user_id: UUID of the user to be cloned.
        """
        if not user_id:
            raise RequiredArgument(
                function=cls.clone_user.__name__, argument="user_id"
            )

        cmd = XmlCommand("create_user")
        cmd.add_element("copy", str(user_id))
        return cmd

    @classmethod
    def delete_user(
        cls,
        user_id: EntityID | None = None,
        *,
        name: str | None = None,
        inheritor_id: EntityID | None = None,
        inheritor_name: str | None = None,
    ) -> Request:
        """Delete an existing user

        Either user_id or name must be passed.

        Args:
            user_id: UUID of the task to be deleted.
            name: The name of the user to be deleted.
            inheritor_id: The UUID of the inheriting user or "self". Overrides
                inheritor_name.
            inheritor_name: The name of the inheriting user.

        """
        if not user_id and not name:
            raise RequiredArgument(
                function=cls.delete_user.__name__, argument="user_id or name"
            )

        cmd = XmlCommand("delete_user")

        if user_id:
            cmd.set_attribute("user_id", str(user_id))

        if name:
            cmd.set_attribute("name", name)

        if inheritor_id:
            cmd.set_attribute("inheritor_id", str(inheritor_id))
        if inheritor_name:
            cmd.set_attribute("inheritor_name", inheritor_name)

        return cmd

    @staticmethod
    def get_users(
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
    ) -> Request:
        """Request a list of users

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
        """
        cmd = XmlCommand("get_users")
        cmd.add_filter(filter_string, filter_id)
        return cmd

    @classmethod
    def get_user(cls, user_id: EntityID) -> Request:
        """Request a single user

        Args:
            user_id: UUID of the user to be requested.
        """
        if not user_id:
            raise RequiredArgument(
                function=cls.get_user.__name__, argument="user_id"
            )

        cmd = XmlCommand("get_users")
        cmd.set_attribute("user_id", str(user_id))
        return cmd



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetUserSettingsTestMixin:
    def test_get_settings(self):
        self.gmp.get_user_settings()

        self.connection.send.has_been_called_with(b"<get_settings/>")

    def test_get_settings_with_filter_string(self):
        self.gmp.get_user_settings(filter_string="foo=bar")

        self.connection.send.has_been_called_with(
            b'<get_settings filter="foo=bar"/>'
        )



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.credentials import (
    GmpCloneCredentialTestMixin,
    GmpCreateCredentialTestMixin,
    GmpDeleteCredentialTestMixin,
    GmpGetCredentialsTestMixin,
    GmpGetCredentialTestMixin,
    GmpModifyCredentialTestMixin,
)
from ...gmpv225 import GMPTestCase


class Gmpv225CloneCredentialTestCase(GmpCloneCredentialTestMixin, GMPTestCase):
    pass


class Gmpv225CreateCredentialTestCase(
    GmpCreateCredentialTestMixin, GMPTestCase
):
    pass


class Gmpv225DeleteCredentialTestCase(
    GmpDeleteCredentialTestMixin, GMPTestCase
):
    pass


class Gmpv225GetCredentialTestCase(GmpGetCredentialTestMixin, GMPTestCase):
    pass


class Gmpv225GetCredentialsTestCase(GmpGetCredentialsTestMixin, GMPTestCase):
    pass


class Gmpv225ModifyCredentialTestCase(
    GmpModifyCredentialTestMixin, GMPTestCase
):
    pass



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm._enum import Enum
from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_bool
from gvm.xml import XmlCommand

from .._entity_id import EntityID


class TicketStatus(Enum):
    """Enum for ticket status"""

    OPEN = "Open"
    FIXED = "Fixed"
    CLOSED = "Closed"


class Tickets:
    @classmethod
    def clone_ticket(cls, ticket_id: EntityID) -> Request:
        """Clone an existing ticket

        Args:
            ticket_id: UUID of an existing ticket to clone from
        """
        if not ticket_id:
            raise RequiredArgument(
                function=cls.clone_ticket.__name__, argument="ticket_id"
            )

        cmd = XmlCommand("create_ticket")

        cmd.add_element("copy", str(ticket_id))

        return cmd

    @classmethod
    def create_ticket(
        cls,
        *,
        result_id: EntityID,
        assigned_to_user_id: EntityID,
        note: str,
        comment: str | None = None,
    ) -> Request:
        """Create a new ticket

        Args:
            result_id: UUID of the result the ticket applies to
            assigned_to_user_id: UUID of a user the ticket should be assigned to
            note: A note about opening the ticket
            comment: Comment for the ticket
        """
        if not result_id:
            raise RequiredArgument(
                function=cls.create_ticket.__name__, argument="result_id"
            )

        if not assigned_to_user_id:
            raise RequiredArgument(
                function=cls.create_ticket.__name__,
                argument="assigned_to_user_id",
            )

        if not note:
            raise RequiredArgument(
                function=cls.create_ticket.__name__, argument="note"
            )

        cmd = XmlCommand("create_ticket")

        result = cmd.add_element("result")
        result.set_attribute("id", str(result_id))

        assigned = cmd.add_element("assigned_to")
        user = assigned.add_element("user")
        user.set_attribute("id", str(assigned_to_user_id))

        cmd.add_element("open_note", note)

        if comment:
            cmd.add_element("comment", comment)

        return cmd

    @classmethod
    def delete_ticket(
        cls, ticket_id: EntityID, *, ultimate: bool | None = False
    ) -> Request:
        """Deletes an existing ticket

        Args:
            ticket_id: UUID of the ticket to be deleted.
            ultimate: Whether to remove entirely, or to the trashcan.
        """
        if not ticket_id:
            raise RequiredArgument(
                function=cls.delete_ticket.__name__, argument="ticket_id"
            )

        cmd = XmlCommand("delete_ticket")
        cmd.set_attribute("ticket_id", str(ticket_id))
        cmd.set_attribute("ultimate", to_bool(ultimate))

        return cmd

    @staticmethod
    def get_tickets(
        *,
        trash: bool | None = None,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
    ) -> Request:
        """Request a list of tickets

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            trash: True to request the tickets in the trashcan
        """
        cmd = XmlCommand("get_tickets")

        cmd.add_filter(filter_string, filter_id)

        if trash is not None:
            cmd.set_attribute("trash", to_bool(trash))

        return cmd

    @classmethod
    def get_ticket(cls, ticket_id: EntityID) -> Request:
        """Request a single ticket

        Args:
            ticket_id: UUID of an existing ticket
        """
        if not ticket_id:
            raise RequiredArgument(
                function=cls.get_ticket.__name__, argument="ticket_id"
            )

        cmd = XmlCommand("get_tickets")
        cmd.set_attribute("ticket_id", str(ticket_id))
        return cmd

    @classmethod
    def modify_ticket(
        cls,
        ticket_id: EntityID,
        *,
        status: TicketStatus | str | None = None,
        note: str | None = None,
        assigned_to_user_id: EntityID | None = None,
        comment: str | None = None,
    ) -> Request:
        """Modify a single ticket

        Args:
            ticket_id: UUID of an existing ticket
            status: New status for the ticket
            note: Note for the status change. Required if status is set.
            assigned_to_user_id: UUID of the user the ticket should be assigned
                to
            comment: Comment for the ticket
        """
        if not ticket_id:
            raise RequiredArgument(
                function=cls.modify_ticket.__name__, argument="ticket_id"
            )

        if status and not note:
            raise RequiredArgument(
                function=cls.modify_ticket.__name__, argument="note"
            )

        if note and not status:
            raise RequiredArgument(
                function=cls.modify_ticket.__name__, argument="status"
            )

        cmd = XmlCommand("modify_ticket")
        cmd.set_attribute("ticket_id", str(ticket_id))

        if assigned_to_user_id:
            assigned = cmd.add_element("assigned_to")
            user = assigned.add_element("user")
            user.set_attribute("id", str(assigned_to_user_id))

        if status:
            if not isinstance(status, TicketStatus):
                status = TicketStatus(status)

            cmd.add_element("status", status.value)
            cmd.add_element(f"{status.name.lower()}_note", note)

        if comment:
            cmd.add_element("comment", comment)

        return cmd



# SPDX-FileCopyrightText: 2019-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import warnings
from collections.abc import Callable
from types import TracebackType

from gvm.__version__ import __version__
from gvm.connections import GvmConnection
from gvm.errors import GvmError
from gvm.protocols.core import Response

from .._protocol import GvmProtocol, T, str_transform
from ._gmp224 import GMPv224
from ._gmp225 import GMPv225
from ._gmp226 import GMPv226
from ._gmp227 import GMPv227
from ._gmpnext import GMPNext
from .requests import Version

SUPPORTED_GMP_VERSIONS = (
    GMPv224[T] | GMPv225[T] | GMPv226[T] | GMPv227[T] | GMPNext[T]
)
_SUPPORTED_GMP_VERSION_STRINGS = ["22.4", "22.5", "22.6", "22.7", "22.8"]


class GMP(GvmProtocol[T]):
    """Dynamically select supported GMP protocol of the remote manager daemon.

    Must be used as a `Context Manager <https://docs.python.org/3/reference/datamodel.html#context-managers>`_

    Example:

        .. code-block:: python

            from gvm.protocols.gmp import GMP

            with GMP(connection) as gmp:
                # gmp can be an instance of
                # gvm.protocols.gmp.GMPv224,
                # gvm.protocols.gmp.GMPv225,
                # gvm.protocols.gmp.GMPv226,
                # gvm.protocols.gmp.GMPv227,
                # or gvm.protocols.gmp.GMPNext
                # depending on the supported GMP version of the remote manager daemon
                resp = gmp.get_tasks()
    """

    def __init__(
        self,
        connection: GvmConnection,
        *,
        transform: Callable[[Response], T] = str_transform,  # type: ignore[assignment]
    ):
        """
        Create a new GMP instance.

        Args:
            connection: Connection to use to talk with the remote daemon. See
                :mod:`gvm.connections` for possible connection types.
            transform: Optional transform `callable <https://docs.python.org/3/library/functions.html#callable>`_
                to convert response data. After each request the callable gets passed the plain response data
                which can be used to check the data and/or conversion into different
                representations like a xml dom.

                See :mod:`gvm.transforms` for existing transforms.
        """
        super().__init__(connection, transform=transform)
        self._gmp: SUPPORTED_GMP_VERSIONS | None = None

    def determine_remote_gmp_version(self) -> str:
        """Determine the supported GMP version of the remote daemon"""
        self.connect()
        resp = self._send_request(Version.get_version())
        self.disconnect()

        version_el = resp.xml().find("version")
        if version_el is None or not version_el.text:
            raise GvmError(
                "Invalid response from manager daemon while requesting the "
                "version information."
            )

        return version_el.text

    def determine_supported_gmp(self) -> SUPPORTED_GMP_VERSIONS:
        """Determine supported GMP version of the remote daemon and return a
        corresponding GMP class instance
        """
        version_str = self.determine_remote_gmp_version().split(".", 1)
        major_version = int(version_str[0])
        minor_version = int(version_str[1])
        if major_version == 22 and minor_version == 4:
            gmp_class = GMPv224
        elif major_version == 22 and minor_version == 5:
            gmp_class = GMPv225
        elif major_version == 22 and minor_version == 6:
            gmp_class = GMPv226
        elif major_version == 22 and minor_version == 7:
            gmp_class = GMPv227
        elif major_version == 22 and minor_version >= 8:
            gmp_class = GMPNext
            if minor_version > 8:
                warnings.warn(
                    "Remote manager daemon uses a newer GMP version than "
                    f"supported by python-gvm {__version__}. Please update to "
                    "a newer release of python-gvm if possible. "
                    f"Remote GMP version is {major_version}.{minor_version}. "
                    f"Supported GMP versions are {', '.join(_SUPPORTED_GMP_VERSION_STRINGS)}."
                )
        else:
            raise GvmError(
                "Remote manager daemon uses an unsupported version of GMP. "
                f"The GMP version was {major_version}.{minor_version}"
                f"Supported GMP versions are {', '.join(_SUPPORTED_GMP_VERSION_STRINGS)}."
            )

        return gmp_class(self._connection, transform=self._transform_callable)  # type: ignore[arg-type]

    def __enter__(self) -> SUPPORTED_GMP_VERSIONS:  # type: ignore[override]
        """
        Returns the corresponding GMP class of the supported GMP version of the
        remote manager daemon.
        """
        self._gmp = self.determine_supported_gmp()

        self._gmp.connect()

        return self._gmp

    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_value: BaseException | None,
        traceback: TracebackType | None,
    ) -> None:
        if self._gmp:
            self._gmp.disconnect()
        self._gmp = None



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetFiltersTestMixin:
    def test_get_filters(self):
        self.gmp.get_filters()

        self.connection.send.has_been_called_with(b"<get_filters/>")

    def test_get_filters_with_filter_string(self):
        self.gmp.get_filters(filter_string="foo=bar")

        self.connection.send.has_been_called_with(
            b'<get_filters filter="foo=bar"/>'
        )

    def test_get_filters_with_filter_id(self):
        self.gmp.get_filters(filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_filters filt_id="f1"/>'
        )

    def test_get_filters_with_trash(self):
        self.gmp.get_filters(trash=True)

        self.connection.send.has_been_called_with(b'<get_filters trash="1"/>')

        self.gmp.get_filters(trash=False)

        self.connection.send.has_been_called_with(b'<get_filters trash="0"/>')

    def test_get_filters_with_alerts(self):
        self.gmp.get_filters(alerts=True)

        self.connection.send.has_been_called_with(b'<get_filters alerts="1"/>')

        self.gmp.get_filters(alerts=False)

        self.connection.send.has_been_called_with(b'<get_filters alerts="0"/>')



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpCreatePolicyTestMixin:
    def test_create_policy(self):
        self.gmp.create_policy("foo")

        self.connection.send.has_been_called_with(
            b"<create_config>"
            b"<copy>085569ce-73ed-11df-83c3-002264764cea</copy>"
            b"<name>foo</name>"
            b"<usage_type>policy</usage_type>"
            b"</create_config>"
        )

    def test_create_with_policy_id_and_comment(self):
        self.gmp.create_policy("foo", policy_id="p1", comment="foo")

        self.connection.send.has_been_called_with(
            b"<create_config>"
            b"<comment>foo</comment>"
            b"<copy>p1</copy>"
            b"<name>foo</name>"
            b"<usage_type>policy</usage_type>"
            b"</create_config>"
        )

    def test_missing_name(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_policy(policy_id="c1", name=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.create_policy(policy_id="c1", name="")



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""
High-level API interface for interacting with openvasd HTTP services via
logical modules (health, metadata, scans, etc.).

Usage:

.. code-block:: python

    from gvm.protocols.http.openvasd import OpenvasdHttpAPIv1
"""

from ._openvasd1 import OpenvasdHttpAPIv1

__all__ = ["OpenvasdHttpAPIv1"]



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .targets import (
    GmpCloneTargetTestMixin,
    GmpCreateTargetTestMixin,
    GmpDeleteTargetTestMixin,
    GmpGetTargetsTestMixin,
    GmpGetTargetTestMixin,
    GmpModifyTargetTestMixin,
)


class Gmpv224CloneTargetTestCase(GmpCloneTargetTestMixin, Gmpv224TestCase):
    pass


class Gmpv224CreateTargetTestCase(GmpCreateTargetTestMixin, Gmpv224TestCase):
    pass


class Gmpv224DeleteTargetTestCase(GmpDeleteTargetTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetTargetTestCase(GmpGetTargetTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetTargetsTestCase(GmpGetTargetsTestMixin, Gmpv224TestCase):
    pass


class Gmpv224ModifyTargetTestCase(GmpModifyTargetTestMixin, Gmpv224TestCase):
    pass



# SPDX-FileCopyrightText: 2023-2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.report_formats import (
    GmpCloneReportFormatTestMixin,
    GmpDeleteReportFormatTestMixin,
    GmpGetReportFormatsTestMixin,
    GmpGetReportFormatTestMixin,
    GmpImportReportFormatTestMixin,
    GmpModifyReportFormatTestMixin,
    GmpVerifyReportFormatTestMixin,
)
from ...gmpv227 import GMPTestCase


class GMPDeleteReportFormatTestCase(
    GmpDeleteReportFormatTestMixin, GMPTestCase
):
    pass


class GMPGetReportFormatTestCase(GmpGetReportFormatTestMixin, GMPTestCase):
    pass


class GMPGetReportFormatsTestCase(GmpGetReportFormatsTestMixin, GMPTestCase):
    pass


class GMPCloneReportFormatTestCase(GmpCloneReportFormatTestMixin, GMPTestCase):
    pass


class GMPImportReportFormatTestCase(
    GmpImportReportFormatTestMixin, GMPTestCase
):
    pass


class GMPModifyReportFormatTestCase(
    GmpModifyReportFormatTestMixin, GMPTestCase
):
    pass


class GMPVerifyReportFormatTestCase(
    GmpVerifyReportFormatTestMixin, GMPTestCase
):
    pass



# SPDX-FileCopyrightText: 2023-2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpnext import GMPTestCase
from ...gmpv224.entities.credentials import (
    GmpCloneCredentialTestMixin,
    GmpCreateCredentialTestMixin,
    GmpDeleteCredentialTestMixin,
    GmpGetCredentialsTestMixin,
    GmpGetCredentialTestMixin,
    GmpModifyCredentialTestMixin,
)
from ..entities.credentials import (
    GmpCreateCredentialStoreCredentialTestMixin,
    GmpModifyCredentialStoreCredentialTestMixin,
)


class GMPCloneCredentialTestCase(GmpCloneCredentialTestMixin, GMPTestCase):
    pass


class GMPCreateCredentialTestCase(
    GmpCreateCredentialTestMixin,
    GmpCreateCredentialStoreCredentialTestMixin,
    GMPTestCase,
):
    pass


class GMPDeleteCredentialTestCase(GmpDeleteCredentialTestMixin, GMPTestCase):
    pass


class GMPGetCredentialTestCase(GmpGetCredentialTestMixin, GMPTestCase):
    pass


class GMPGetCredentialsTestCase(GmpGetCredentialsTestMixin, GMPTestCase):
    pass


class GMPModifyCredentialTestCase(
    GmpModifyCredentialTestMixin,
    GmpModifyCredentialStoreCredentialTestMixin,
    GMPTestCase,
):
    pass



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from gvm.errors import InvalidArgument
from gvm.protocols.gmp.requests.v225 import AlertEvent


class GetAlertEventFromStringTestCase(unittest.TestCase):
    def test_invalid(self):
        with self.assertRaises(InvalidArgument):
            AlertEvent.from_string("foo")

    def test_none_or_empty(self):
        ct = AlertEvent.from_string(None)
        self.assertIsNone(ct)
        ct = AlertEvent.from_string("")
        self.assertIsNone(ct)

    def test_task_run_status_changed(self):
        ct = AlertEvent.from_string("Task run status changed")
        self.assertEqual(ct, AlertEvent.TASK_RUN_STATUS_CHANGED)

    def test_new_secinfo_arrived(self):
        ct = AlertEvent.from_string("New SecInfo arrived")
        self.assertEqual(ct, AlertEvent.NEW_SECINFO_ARRIVED)

    def test_updated_secinfo_arrived(self):
        ct = AlertEvent.from_string("Updated SecInfo arrived")
        self.assertEqual(ct, AlertEvent.UPDATED_SECINFO_ARRIVED)

    def test_ticket_received(self):
        ct = AlertEvent.from_string("ticket received")
        self.assertEqual(ct, AlertEvent.TICKET_RECEIVED)

    def test_assigned_ticket_changed(self):
        ct = AlertEvent.from_string("assigned ticket changed")
        self.assertEqual(ct, AlertEvent.ASSIGNED_TICKET_CHANGED)

    def test_owned_ticket_changed(self):
        ct = AlertEvent.from_string("owned ticket changed")
        self.assertEqual(ct, AlertEvent.OWNED_TICKET_CHANGED)


if __name__ == "__main__":
    unittest.main()



#  SPDX-FileCopyrightText: 2025 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpCreateOCIImageTargetTestMixin:
    def test_create_target_missing_name(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_oci_image_target(
                None, image_references=["oci:foo/bar:latest"]
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.create_oci_image_target(
                name=None, image_references=["oci:foo/bar:latest"]
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.create_oci_image_target(
                "", image_references=["oci:foo/bar:latest"]
            )

    def test_create_target_missing_image_references(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_oci_image_target(name="foo", image_references=None)

    def test_create_target_with_comment(self):
        self.gmp.create_oci_image_target(
            "foo", image_references=["oci:foo/bar:latest"], comment="bar"
        )

        self.connection.send.has_been_called_with(
            b"<create_oci_image_target>"
            b"<name>foo</name>"
            b"<image_references>oci:foo/bar:latest</image_references>"
            b"<comment>bar</comment>"
            b"</create_oci_image_target>"
        )

    def test_create_target_with_smb_credential_id(self):
        self.gmp.create_oci_image_target(
            "foo", image_references=["oci:foo/bar:latest"], credential_id="c1"
        )

        self.connection.send.has_been_called_with(
            b"<create_oci_image_target>"
            b"<name>foo</name>"
            b"<image_references>oci:foo/bar:latest</image_references>"
            b'<credential id="c1"/>'
            b"</create_oci_image_target>"
        )



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .scanners import (
    GmpCloneScannerTestMixin,
    GmpCreateScannerTestMixin,
    GmpDeleteScannerTestMixin,
    GmpGetScannersTestMixin,
    GmpGetScannerTestMixin,
    GmpModifyScannerTestMixin,
    GmpVerifyScannerTestMixin,
)


class Gmpv224DeleteScannerTestCase(GmpDeleteScannerTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetScannerTestCase(GmpGetScannerTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetScannersTestCase(GmpGetScannersTestMixin, Gmpv224TestCase):
    pass


class Gmpv224CloneScannerTestCase(GmpCloneScannerTestMixin, Gmpv224TestCase):
    pass


class Gmpv224CreateScannerTestCase(GmpCreateScannerTestMixin, Gmpv224TestCase):
    pass


class Gmpv224ModifyScannerTestCase(GmpModifyScannerTestMixin, Gmpv224TestCase):
    pass


class Gmpv224VerifyScannerTestCase(GmpVerifyScannerTestMixin, Gmpv224TestCase):
    pass



# SPDX-FileCopyrightText: 2021-2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from typing import Optional

from gvm._enum import Enum
from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_bool
from gvm.xml import XmlCommand

from .._entity_id import EntityID


class ScannerType(Enum):
    """Enum for scanner type"""

    # 1 was removed (OSP_SCANNER_TYPE).
    OPENVAS_SCANNER_TYPE = "2"
    CVE_SCANNER_TYPE = "3"
    GREENBONE_SENSOR_SCANNER_TYPE = "5"
    OPENVASD_SCANNER_TYPE = "6"

    @classmethod
    def from_string(
        cls,
        scanner_type: str | None,
    ) -> Optional["ScannerType"]:
        """Convert a scanner type string to an actual ScannerType instance

        Arguments:
            scanner_type: Scanner type string to convert to a ScannerType
        """
        if not scanner_type:
            return None

        scanner_type = scanner_type.lower()

        if (
            scanner_type == cls.OPENVAS_SCANNER_TYPE.value
            or scanner_type == "openvas"
            or scanner_type == "openvas_scanner_type"
        ):
            return cls.OPENVAS_SCANNER_TYPE

        if (
            scanner_type == cls.CVE_SCANNER_TYPE.value
            or scanner_type == "cve"
            or scanner_type == "cve_scanner_type"
        ):
            return cls.CVE_SCANNER_TYPE

        if (
            scanner_type == cls.GREENBONE_SENSOR_SCANNER_TYPE.value
            or scanner_type == "greenbone"
            or scanner_type == "greenbone_sensor_scanner_type"
        ):
            return cls.GREENBONE_SENSOR_SCANNER_TYPE

        if (
            scanner_type == cls.OPENVASD_SCANNER_TYPE.value
            or scanner_type == "openvasd"
            or scanner_type == "openvasd_scanner_type"
        ):
            return cls.OPENVASD_SCANNER_TYPE

        raise InvalidArgument(
            argument="scanner_type", function=cls.from_string.__name__
        )


class Scanners:
    @classmethod
    def create_scanner(
        cls,
        name: str,
        host: str,
        port: str | int,
        scanner_type: ScannerType,
        credential_id: str,
        *,
        ca_pub: str | None = None,
        comment: str | None = None,
        relay_host: str | None = None,
        relay_port: str | int | None = None,
    ) -> Request:
        """Create a new scanner

        Args:
            name: Name of the new scanner
            host: Hostname or IP address of the scanner
            port: Port of the scanner
            scanner_type: Type of the scanner
            credential_id: UUID of client certificate credential for the
                scanner
            ca_pub: Certificate of CA to verify scanner certificate
            comment: Comment for the scanner
            relay_host: Hostname or IP address of the scanner relay
            relay_port: Port of the scanner relay
        """
        if not name:
            raise RequiredArgument(
                function=cls.create_scanner.__name__, argument="name"
            )

        if not host:
            raise RequiredArgument(
                function=cls.create_scanner.__name__, argument="host"
            )

        if not port:
            raise RequiredArgument(
                function=cls.create_scanner.__name__, argument="port"
            )

        if not scanner_type:
            raise RequiredArgument(
                function=cls.create_scanner.__name__, argument="scanner_type"
            )

        if not credential_id:
            raise RequiredArgument(
                function=cls.create_scanner.__name__, argument="credential_id"
            )

        cmd = XmlCommand("create_scanner")
        cmd.add_element("name", name)
        cmd.add_element("host", host)
        cmd.add_element("port", str(port))

        if not isinstance(scanner_type, ScannerType):
            scanner_type = ScannerType(scanner_type)

        cmd.add_element("type", scanner_type.value)

        cmd.add_element("credential", attrs={"id": str(credential_id)})

        if ca_pub:
            cmd.add_element("ca_pub", ca_pub)

        if comment:
            cmd.add_element("comment", comment)

        if relay_host:
            cmd.add_element("relay_host", relay_host)

        if relay_port:
            cmd.add_element("relay_port", str(relay_port))

        return cmd

    @classmethod
    def modify_scanner(
        cls,
        scanner_id: EntityID,
        *,
        name: str | None = None,
        host: str | None = None,
        port: int | None = None,
        scanner_type: ScannerType | None = None,
        credential_id: EntityID | None = None,
        ca_pub: str | None = None,
        comment: str | None = None,
        relay_host: str | None = None,
        relay_port: str | int | None = None,
    ) -> Request:
        """Modify an existing scanner

        Args:
            scanner_id: UUID of the scanner to modify
            name: New name of the scanner
            host: New hostname or IP address of the scanner
            port: New port of the scanner
            scanner_type: New type of the scanner
            credential_id: New UUID of client certificate credential for the
                scanner
            ca_pub: New certificate of CA to verify scanner certificate
            comment: New comment for the scanner
            relay_host: Hostname or IP address of the scanner relay
            relay_port: Port of the scanner relay
        """
        if not scanner_id:
            raise RequiredArgument(
                function=cls.modify_scanner.__name__, argument="scanner_id"
            )

        cmd = XmlCommand("modify_scanner")
        cmd.set_attribute("scanner_id", str(scanner_id))

        if scanner_type is not None:
            if not isinstance(scanner_type, ScannerType):
                scanner_type = ScannerType(scanner_type)
            if not scanner_type:
                raise InvalidArgument(
                    argument="scanner_type",
                    function=cls.modify_scanner.__name__,
                )
            cmd.add_element("type", scanner_type.value)

        if host:
            cmd.add_element("host", host)

        if port:
            cmd.add_element("port", str(port))

        if comment:
            cmd.add_element("comment", comment)

        if name:
            cmd.add_element("name", name)

        if ca_pub:
            cmd.add_element("ca_pub", ca_pub)

        if credential_id:
            cmd.add_element("credential", attrs={"id": str(credential_id)})

        if relay_host:
            cmd.add_element("relay_host", relay_host)

        if relay_port:
            cmd.add_element("relay_port", str(relay_port))

        return cmd

    @staticmethod
    def get_scanners(
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        trash: bool | None = None,
        details: bool | None = None,
    ) -> Request:
        """Request a list of scanners

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            trash: Whether to get the trashcan scanners instead
            details: Whether to include extra details like tasks using this
                scanner
        """
        cmd = XmlCommand("get_scanners")
        cmd.add_filter(filter_string, filter_id)

        if trash is not None:
            cmd.set_attribute("trash", to_bool(trash))

        if details is not None:
            cmd.set_attribute("details", to_bool(details))

        return cmd

    @classmethod
    def get_scanner(cls, scanner_id: EntityID) -> Request:
        """Request a single scanner

        Args:
            scanner_id: UUID of an existing scanner
        """
        if not scanner_id:
            raise RequiredArgument(
                function=cls.get_scanner.__name__, argument="scanner_id"
            )

        cmd = XmlCommand("get_scanners")
        cmd.set_attribute("scanner_id", str(scanner_id))

        # for single entity always request all details
        cmd.set_attribute("details", "1")

        return cmd

    @classmethod
    def verify_scanner(cls, scanner_id: EntityID) -> Request:
        """Verify an existing scanner

        Args:
            scanner_id: UUID of an existing scanner
        """
        if not scanner_id:
            raise RequiredArgument(
                function=cls.verify_scanner.__name__, argument="scanner_id"
            )

        cmd = XmlCommand("verify_scanner")
        cmd.set_attribute("scanner_id", str(scanner_id))

        return cmd

    @classmethod
    def clone_scanner(cls, scanner_id: EntityID) -> Request:
        """Clone an existing scanner

        Args:
            scanner_id: UUID of an existing scanner
        """
        if not scanner_id:
            raise RequiredArgument(
                function=cls.clone_scanner.__name__, argument="scanner_id"
            )

        cmd = XmlCommand("create_scanner")
        cmd.add_element("copy", str(scanner_id))
        return cmd

    @classmethod
    def delete_scanner(
        cls, scanner_id: EntityID, ultimate: bool | None = False
    ) -> Request:
        """Delete an existing scanner

        Args:
            scanner_id: UUID of an existing scanner
        """
        if not scanner_id:
            raise RequiredArgument(
                function=cls.delete_scanner.__name__, argument="scanner_id"
            )

        cmd = XmlCommand("delete_scanner")
        cmd.set_attribute("scanner_id", str(scanner_id))
        cmd.set_attribute("ultimate", to_bool(ultimate))

        return cmd



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import RequiredArgument
from gvm.protocols.gmp.requests.v224 import TLSCertificates


class TLSCertificatesTestCase(unittest.TestCase):
    def test_clone_tls_certificate(self):
        request = TLSCertificates.clone_tls_certificate("tls_certificate_id")
        self.assertEqual(
            bytes(request),
            b"<create_tls_certificate>"
            b"<copy>tls_certificate_id</copy>"
            b"</create_tls_certificate>",
        )

    def test_clone_tls_certificate_missing_tls_certificate_id(self):
        with self.assertRaises(RequiredArgument):
            TLSCertificates.clone_tls_certificate(None)

        with self.assertRaises(RequiredArgument):
            TLSCertificates.clone_tls_certificate("")

    def test_create_tls_certificate(self):
        request = TLSCertificates.create_tls_certificate(
            name="name",
            certificate="certificate",
        )
        self.assertEqual(
            bytes(request),
            b"<create_tls_certificate>"
            b"<name>name</name>"
            b"<certificate>certificate</certificate>"
            b"</create_tls_certificate>",
        )

    def test_create_tls_certificate_with_comment(self):
        request = TLSCertificates.create_tls_certificate(
            name="name",
            certificate="certificate",
            comment="comment",
        )
        self.assertEqual(
            bytes(request),
            b"<create_tls_certificate>"
            b"<name>name</name>"
            b"<certificate>certificate</certificate>"
            b"<comment>comment</comment>"
            b"</create_tls_certificate>",
        )

    def test_create_tls_certificate_with_trust(self):
        request = TLSCertificates.create_tls_certificate(
            name="name",
            certificate="certificate",
            trust=True,
        )
        self.assertEqual(
            bytes(request),
            b"<create_tls_certificate>"
            b"<name>name</name>"
            b"<certificate>certificate</certificate>"
            b"<trust>1</trust>"
            b"</create_tls_certificate>",
        )

        request = TLSCertificates.create_tls_certificate(
            name="name",
            certificate="certificate",
            trust=False,
        )
        self.assertEqual(
            bytes(request),
            b"<create_tls_certificate>"
            b"<name>name</name>"
            b"<certificate>certificate</certificate>"
            b"<trust>0</trust>"
            b"</create_tls_certificate>",
        )

    def test_create_tls_certificate_missing_name(self):
        with self.assertRaises(RequiredArgument):
            TLSCertificates.create_tls_certificate(
                name=None,
                certificate="certificate",
            )

        with self.assertRaises(RequiredArgument):
            TLSCertificates.create_tls_certificate(
                name="",
                certificate="certificate",
            )

    def test_create_tls_certificate_missing_certificate(self):
        with self.assertRaises(RequiredArgument):
            TLSCertificates.create_tls_certificate(
                name="name",
                certificate=None,
            )

        with self.assertRaises(RequiredArgument):
            TLSCertificates.create_tls_certificate(
                name="name",
                certificate="",
            )

    def test_delete_tls_certificate(self):
        request = TLSCertificates.delete_tls_certificate("tls_certificate_id")
        self.assertEqual(
            bytes(request),
            b'<delete_tls_certificate tls_certificate_id="tls_certificate_id"/>',
        )

    def test_delete_tls_certificate_missing_tls_certificate_id(self):
        with self.assertRaises(RequiredArgument):
            TLSCertificates.delete_tls_certificate(None)

        with self.assertRaises(RequiredArgument):
            TLSCertificates.delete_tls_certificate("")

    def test_get_tls_certificates(self):
        request = TLSCertificates.get_tls_certificates()
        self.assertEqual(
            bytes(request),
            b"<get_tls_certificates/>",
        )

    def test_get_tls_certificates_with_filter_string(self):
        request = TLSCertificates.get_tls_certificates(
            filter_string="filter_string"
        )
        self.assertEqual(
            bytes(request),
            b'<get_tls_certificates filter="filter_string"/>',
        )

    def test_get_tls_certificates_with_filter_id(self):
        request = TLSCertificates.get_tls_certificates(filter_id="filter_id")
        self.assertEqual(
            bytes(request),
            b'<get_tls_certificates filt_id="filter_id"/>',
        )

    def test_get_tls_certificates_with_details(self):
        request = TLSCertificates.get_tls_certificates(details=True)
        self.assertEqual(
            bytes(request),
            b'<get_tls_certificates details="1"/>',
        )

        request = TLSCertificates.get_tls_certificates(details=False)
        self.assertEqual(
            bytes(request),
            b'<get_tls_certificates details="0"/>',
        )

    def test_get_tls_certificates_with_include_certificate_data(self):
        request = TLSCertificates.get_tls_certificates(
            include_certificate_data=True
        )
        self.assertEqual(
            bytes(request),
            b'<get_tls_certificates include_certificate_data="1"/>',
        )

        request = TLSCertificates.get_tls_certificates(
            include_certificate_data=False
        )
        self.assertEqual(
            bytes(request),
            b'<get_tls_certificates include_certificate_data="0"/>',
        )

    def test_get_tls_certificate(self):
        request = TLSCertificates.get_tls_certificate("tls_certificate_id")
        self.assertEqual(
            bytes(request),
            b'<get_tls_certificates tls_certificate_id="tls_certificate_id" include_certificate_data="1" details="1"/>',
        )

    def test_get_tls_certificate_missing_tls_certificate_id(self):
        with self.assertRaises(RequiredArgument):
            TLSCertificates.get_tls_certificate(None)

        with self.assertRaises(RequiredArgument):
            TLSCertificates.get_tls_certificate("")

    def test_modify_tls_certificate(self):
        request = TLSCertificates.modify_tls_certificate(
            "tls_certificate_id",
        )
        self.assertEqual(
            bytes(request),
            b'<modify_tls_certificate tls_certificate_id="tls_certificate_id"/>',
        )

    def test_modify_tls_certificate_with_name(self):
        request = TLSCertificates.modify_tls_certificate(
            "tls_certificate_id",
            name="name",
        )
        self.assertEqual(
            bytes(request),
            b'<modify_tls_certificate tls_certificate_id="tls_certificate_id">'
            b"<name>name</name>"
            b"</modify_tls_certificate>",
        )

    def test_modify_tls_certificate_with_comment(self):
        request = TLSCertificates.modify_tls_certificate(
            "tls_certificate_id",
            comment="comment",
        )
        self.assertEqual(
            bytes(request),
            b'<modify_tls_certificate tls_certificate_id="tls_certificate_id">'
            b"<comment>comment</comment>"
            b"</modify_tls_certificate>",
        )

    def test_modify_tls_certificate_with_trust(self):
        request = TLSCertificates.modify_tls_certificate(
            "tls_certificate_id",
            trust=True,
        )
        self.assertEqual(
            bytes(request),
            b'<modify_tls_certificate tls_certificate_id="tls_certificate_id">'
            b"<trust>1</trust>"
            b"</modify_tls_certificate>",
        )

        request = TLSCertificates.modify_tls_certificate(
            "tls_certificate_id",
            trust=False,
        )
        self.assertEqual(
            bytes(request),
            b'<modify_tls_certificate tls_certificate_id="tls_certificate_id">'
            b"<trust>0</trust>"
            b"</modify_tls_certificate>",
        )



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm._enum import Enum
from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_bool
from gvm.xml import XmlCommand

from .._entity_id import EntityID


class InfoType(Enum):
    """Enum for info types"""

    CERT_BUND_ADV = "CERT_BUND_ADV"
    CPE = "CPE"
    CVE = "CVE"
    DFN_CERT_ADV = "DFN_CERT_ADV"
    OVALDEF = "OVALDEF"
    NVT = "NVT"


class SecInfo:
    @classmethod
    def get_info(cls, info_id: EntityID, info_type: InfoType) -> Request:
        """Request a single secinfo

        Arguments:
            info_id: ID of an existing secinfo
            info_type: Type must be either CERT_BUND_ADV, CPE, CVE,
                DFN_CERT_ADV, OVALDEF, NVT
        """
        if not info_type:
            raise RequiredArgument(
                function=cls.get_info.__name__, argument="info_type"
            )

        if not isinstance(info_type, InfoType):
            info_type = InfoType(info_type)

        if not info_id:
            raise RequiredArgument(
                function=cls.get_info.__name__, argument="info_id"
            )

        cmd = XmlCommand("get_info")
        cmd.set_attribute("info_id", str(info_id))

        cmd.set_attribute("type", info_type.value)

        # for single entity always request all details
        cmd.set_attribute("details", "1")
        return cmd

    @classmethod
    def get_info_list(
        cls,
        info_type: InfoType,
        *,
        filter_string: str | None = None,
        filter_id: str | None = None,
        name: str | None = None,
        details: bool | None = None,
    ) -> Request:
        """Request a list of security information

        Args:
            info_type: Type must be either CERT_BUND_ADV, CPE, CVE,
                DFN_CERT_ADV, OVALDEF or NVT
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            name: Name or identifier of the requested information
            details: Whether to include information about references to this
                information
        """
        if not info_type:
            raise RequiredArgument(
                function=cls.get_info_list.__name__, argument="info_type"
            )

        if not isinstance(info_type, InfoType):
            info_type = InfoType(info_type)

        cmd = XmlCommand("get_info")

        cmd.set_attribute("type", info_type.value)

        cmd.add_filter(filter_string, filter_id)

        if name:
            cmd.set_attribute("name", name)

        if details is not None:
            cmd.set_attribute("details", to_bool(details))

        return cmd



# SPDX-FileCopyrightText: 2020-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import FilterType


class GmpModifyFilterTestMixin:
    def test_modify_filter(self):
        self.gmp.modify_filter(filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<modify_filter filter_id="f1"/>'
        )

    def test_modify_filter_with_filter_type(self):
        self.gmp.modify_filter(filter_id="f1", filter_type=FilterType.TASK)

        self.connection.send.has_been_called_with(
            b'<modify_filter filter_id="f1"><type>task</type></modify_filter>'
        )

    def test_modify_filter_invalid_filter_type(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.modify_filter(filter_id="f1", filter_type="foo")

    def test_modify_filter_missing_filter_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_filter(filter_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_filter(filter_id="")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_filter("")

    def test_modify_filter_with_comment(self):
        self.gmp.modify_filter(filter_id="f1", comment="foo")

        self.connection.send.has_been_called_with(
            b'<modify_filter filter_id="f1">'
            b"<comment>foo</comment>"
            b"</modify_filter>"
        )

    def test_modify_filter_with_name(self):
        self.gmp.modify_filter(filter_id="f1", name="foo")

        self.connection.send.has_been_called_with(
            b'<modify_filter filter_id="f1"><name>foo</name></modify_filter>'
        )

    def test_modify_filter_with_term(self):
        self.gmp.modify_filter(filter_id="f1", term="foo=bar")

        self.connection.send.has_been_called_with(
            b'<modify_filter filter_id="f1">'
            b"<term>foo=bar</term>"
            b"</modify_filter>"
        )



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import InvalidArgument, InvalidArgumentType, RequiredArgument
from gvm.protocols.gmp.requests.v224 import ScanConfigs


class ScanConfigsTestCase(unittest.TestCase):
    def test_clone_scan_config(self):
        request = ScanConfigs.clone_scan_config("a1")

        self.assertEqual(
            bytes(request), b"<create_config><copy>a1</copy></create_config>"
        )

    def test_clone_scan_config_missing_id(self):
        with self.assertRaises(RequiredArgument):
            ScanConfigs.clone_scan_config("")

        with self.assertRaises(RequiredArgument):
            ScanConfigs.clone_scan_config(None)

    def test_create_scan_config(self):
        request = ScanConfigs.create_scan_config("a1", "foo")

        self.assertEqual(
            bytes(request),
            b"<create_config><copy>a1</copy><name>foo</name>"
            b"<usage_type>scan</usage_type></create_config>",
        )

    def test_create_scan_config_with_comment(self):
        request = ScanConfigs.create_scan_config("a1", "foo", comment="comment")

        self.assertEqual(
            bytes(request),
            b"<create_config><comment>comment</comment><copy>a1</copy>"
            b"<name>foo</name><usage_type>scan</usage_type></create_config>",
        )

    def test_create_scan_config_missing_id(self):
        with self.assertRaises(RequiredArgument):
            ScanConfigs.create_scan_config("", "foo")

        with self.assertRaises(RequiredArgument):
            ScanConfigs.create_scan_config(None, "foo")

    def test_create_scan_config_missing_name(self):
        with self.assertRaises(RequiredArgument):
            ScanConfigs.create_scan_config("c1", None)

        with self.assertRaises(RequiredArgument):
            ScanConfigs.create_scan_config("c1", "")

    def test_delete_scan_config(self):
        request = ScanConfigs.delete_scan_config("a1")

        self.assertEqual(
            bytes(request), b'<delete_config config_id="a1" ultimate="0"/>'
        )

    def test_delete_scan_config_ultimate(self):
        request = ScanConfigs.delete_scan_config("a1", ultimate=True)

        self.assertEqual(
            bytes(request), b'<delete_config config_id="a1" ultimate="1"/>'
        )

    def test_delete_scan_config_missing_id(self):
        with self.assertRaises(RequiredArgument):
            ScanConfigs.delete_scan_config("")

        with self.assertRaises(RequiredArgument):
            ScanConfigs.delete_scan_config(None)

    def test_get_scan_config_preference(self):
        request = ScanConfigs.get_scan_config_preference("foo")

        self.assertEqual(bytes(request), b'<get_preferences preference="foo"/>')

    def test_get_scan_config_preference_missing_name(self):
        with self.assertRaises(RequiredArgument):
            ScanConfigs.get_scan_config_preference("")

        with self.assertRaises(RequiredArgument):
            ScanConfigs.get_scan_config_preference(None)

    def test_get_scan_config_preference_with_nvt_oid(self):
        request = ScanConfigs.get_scan_config_preference("foo", nvt_oid="oid")

        self.assertEqual(
            bytes(request), b'<get_preferences preference="foo" nvt_oid="oid"/>'
        )

    def test_get_scan_config_preference_with_config_id(self):
        request = ScanConfigs.get_scan_config_preference("foo", config_id="c1")

        self.assertEqual(
            bytes(request),
            b'<get_preferences preference="foo" config_id="c1"/>',
        )

    def test_get_scan_config_preferences(self):
        request = ScanConfigs.get_scan_config_preferences()

        self.assertEqual(bytes(request), b"<get_preferences/>")

    def test_get_scan_config_preferences_with_nvt_oid(self):
        request = ScanConfigs.get_scan_config_preferences(nvt_oid="oid")

        self.assertEqual(bytes(request), b'<get_preferences nvt_oid="oid"/>')

    def test_get_scan_config_preferences_with_config_id(self):
        request = ScanConfigs.get_scan_config_preferences(config_id="c1")

        self.assertEqual(bytes(request), b'<get_preferences config_id="c1"/>')

    def test_get_scan_config(self):
        request = ScanConfigs.get_scan_config("a1")

        self.assertEqual(
            bytes(request),
            b'<get_configs config_id="a1" usage_type="scan" details="1"/>',
        )

    def test_get_scan_config_with_tasks(self):
        request = ScanConfigs.get_scan_config("a1", tasks=True)

        self.assertEqual(
            bytes(request),
            b'<get_configs config_id="a1" usage_type="scan" tasks="1" details="1"/>',
        )

    def test_get_scan_config_without_scan_config_id(self):
        with self.assertRaises(RequiredArgument):
            ScanConfigs.get_scan_config(None)

        with self.assertRaises(RequiredArgument):
            ScanConfigs.get_scan_config("")

    def test_get_scan_configs(self):
        request = ScanConfigs.get_scan_configs()

        self.assertEqual(bytes(request), b'<get_configs usage_type="scan"/>')

    def test_get_scan_configs_with_filter_string(self):
        request = ScanConfigs.get_scan_configs(filter_string="name=foo")

        self.assertEqual(
            bytes(request),
            b'<get_configs usage_type="scan" filter="name=foo"/>',
        )

    def test_get_scan_configs_with_filter_id(self):
        request = ScanConfigs.get_scan_configs(filter_id="f1")

        self.assertEqual(
            bytes(request),
            b'<get_configs usage_type="scan" filt_id="f1"/>',
        )

    def test_get_scan_configs_from_trash(self):
        request = ScanConfigs.get_scan_configs(trash=True)

        self.assertEqual(
            bytes(request),
            b'<get_configs usage_type="scan" trash="1"/>',
        )

    def test_get_scan_configs_with_details(self):
        request = ScanConfigs.get_scan_configs(details=True)

        self.assertEqual(
            bytes(request),
            b'<get_configs usage_type="scan" details="1"/>',
        )

    def test_get_scan_configs_without_details(self):
        request = ScanConfigs.get_scan_configs(details=False)

        self.assertEqual(
            bytes(request),
            b'<get_configs usage_type="scan" details="0"/>',
        )

    def test_get_scan_configs_with_families(self):
        request = ScanConfigs.get_scan_configs(families=True)

        self.assertEqual(
            bytes(request),
            b'<get_configs usage_type="scan" families="1"/>',
        )

    def test_get_scan_configs_without_families(self):
        request = ScanConfigs.get_scan_configs(families=False)

        self.assertEqual(
            bytes(request),
            b'<get_configs usage_type="scan" families="0"/>',
        )

    def test_get_scan_configs_with_preferences(self):
        request = ScanConfigs.get_scan_configs(preferences=True)

        self.assertEqual(
            bytes(request),
            b'<get_configs usage_type="scan" preferences="1"/>',
        )

    def test_get_scan_configs_without_preferences(self):
        request = ScanConfigs.get_scan_configs(preferences=False)

        self.assertEqual(
            bytes(request),
            b'<get_configs usage_type="scan" preferences="0"/>',
        )

    def test_get_scan_configs_with_tasks(self):
        request = ScanConfigs.get_scan_configs(tasks=True)

        self.assertEqual(
            bytes(request),
            b'<get_configs usage_type="scan" tasks="1"/>',
        )

    def test_get_scan_configs_without_tasks(self):
        request = ScanConfigs.get_scan_configs(tasks=False)

        self.assertEqual(
            bytes(request),
            b'<get_configs usage_type="scan" tasks="0"/>',
        )

    def test_import_scan_config(self):
        config_xml_string = (
            '<get_configs_response status="200" status_text="OK">'
            '<config id="c4aa21e4-23e6-4064-ae49-c0d425738a98">'
            "<name>Foobar</name>"
            "<comment>Foobar config</comment>"
            "<creation_time>2018-11-09T10:48:03Z</creation_time>"
            "<modification_time>2018-11-09T10:48:03Z</modification_time>"
            "</config>"
            "</get_configs_response>"
        )
        request = ScanConfigs.import_scan_config(config_xml_string)

        self.assertEqual(
            bytes(request),
            f"<create_config>{config_xml_string}</create_config>".encode(),
        )

    def test_import_missing_scan_config_xml(self):
        with self.assertRaises(RequiredArgument):
            ScanConfigs.import_scan_config(None)

        with self.assertRaises(RequiredArgument):
            ScanConfigs.import_scan_config("")

    def test_import_invalid_xml(self):
        with self.assertRaises(InvalidArgument):
            ScanConfigs.import_scan_config("abcdef")

    def test_modify_scan_config_set_comment(self):
        request = ScanConfigs.modify_scan_config_set_comment("c1")

        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="c1"><comment></comment></modify_config>',
        )

        request = ScanConfigs.modify_scan_config_set_comment(
            "c1", comment="foo"
        )

        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="c1"><comment>foo</comment></modify_config>',
        )

        request = ScanConfigs.modify_scan_config_set_comment("c1", comment=None)

        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="c1"><comment></comment></modify_config>',
        )

    def test_modify_scan_config_set_comment_missing_config_id(self):
        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_comment(config_id=None)

        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_comment("")

        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_comment(config_id="")

    def test_modify_scan_config_set_family_selection(self):
        request = ScanConfigs.modify_scan_config_set_family_selection(
            "c1", [("foo", True, True)]
        )

        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="c1">'
            b"<family_selection>"
            b"<growing>1</growing>"
            b"<family>"
            b"<name>foo</name>"
            b"<all>1</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"</family_selection>"
            b"</modify_config>",
        )

        request = ScanConfigs.modify_scan_config_set_family_selection(
            "c1", [("foo", True, True), ("bar", True, True)]
        )

        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="c1">'
            b"<family_selection>"
            b"<growing>1</growing>"
            b"<family>"
            b"<name>foo</name>"
            b"<all>1</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"<family>"
            b"<name>bar</name>"
            b"<all>1</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"</family_selection>"
            b"</modify_config>",
        )

        request = ScanConfigs.modify_scan_config_set_family_selection(
            "c1", (("foo", True, True), ("bar", True, True))
        )

        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="c1">'
            b"<family_selection>"
            b"<growing>1</growing>"
            b"<family>"
            b"<name>foo</name>"
            b"<all>1</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"<family>"
            b"<name>bar</name>"
            b"<all>1</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"</family_selection>"
            b"</modify_config>",
        )

        request = ScanConfigs.modify_scan_config_set_family_selection(
            "c1", [("foo", True, False), ("bar", False, True)]
        )

        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="c1">'
            b"<family_selection>"
            b"<growing>1</growing>"
            b"<family>"
            b"<name>foo</name>"
            b"<all>0</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"<family>"
            b"<name>bar</name>"
            b"<all>1</all>"
            b"<growing>0</growing>"
            b"</family>"
            b"</family_selection>"
            b"</modify_config>",
        )

    def test_modify_scan_config_set_family_selection_missing_config_id(self):
        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_family_selection(
                None, [("foo", True, True)]
            )

        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_family_selection(
                "", [("foo", True, True)]
            )

    def test_modify_scan_config_set_family_selection_invalid_families(self):
        with self.assertRaises(InvalidArgumentType):
            ScanConfigs.modify_scan_config_set_family_selection(
                "c1",
                None,
            )

        with self.assertRaises(InvalidArgumentType):
            ScanConfigs.modify_scan_config_set_family_selection(
                "c1",
                "",
            )

    def test_modify_scan_config_set_family_selection_with_auto_add_new_families(
        self,
    ):
        request = ScanConfigs.modify_scan_config_set_family_selection(
            "c1", [("foo", True, True)], auto_add_new_families=True
        )

        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="c1">'
            b"<family_selection>"
            b"<growing>1</growing>"
            b"<family>"
            b"<name>foo</name>"
            b"<all>1</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"</family_selection>"
            b"</modify_config>",
        )

        request = ScanConfigs.modify_scan_config_set_family_selection(
            "c1", [("foo", True, True)], auto_add_new_families=False
        )

        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="c1">'
            b"<family_selection>"
            b"<growing>0</growing>"
            b"<family>"
            b"<name>foo</name>"
            b"<all>1</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"</family_selection>"
            b"</modify_config>",
        )

    def test_modify_scan_config_set_family_selection_with_auto_add_new_nvts(
        self,
    ):
        request = ScanConfigs.modify_scan_config_set_family_selection(
            "c1", [("foo", True, True)]
        )

        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="c1">'
            b"<family_selection>"
            b"<growing>1</growing>"
            b"<family>"
            b"<name>foo</name>"
            b"<all>1</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"</family_selection>"
            b"</modify_config>",
        )

        request = ScanConfigs.modify_scan_config_set_family_selection(
            "c1", [("foo", False, True)]
        )

        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="c1">'
            b"<family_selection>"
            b"<growing>1</growing>"
            b"<family>"
            b"<name>foo</name>"
            b"<all>1</all>"
            b"<growing>0</growing>"
            b"</family>"
            b"</family_selection>"
            b"</modify_config>",
        )

        request = ScanConfigs.modify_scan_config_set_family_selection(
            "c1",
            [("foo", False, True), ("bar", True, False)],
        )

        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="c1">'
            b"<family_selection>"
            b"<growing>1</growing>"
            b"<family>"
            b"<name>foo</name>"
            b"<all>1</all>"
            b"<growing>0</growing>"
            b"</family>"
            b"<family>"
            b"<name>bar</name>"
            b"<all>0</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"</family_selection>"
            b"</modify_config>",
        )

    def test_modify_scan_config_set_family_selection_with_invalid_arguments(
        self,
    ):
        with self.assertRaises(InvalidArgumentType):
            ScanConfigs.modify_scan_config_set_family_selection(
                "c1", [("foo", "False", "True")]
            )

        with self.assertRaises(InvalidArgumentType):
            ScanConfigs.modify_scan_config_set_family_selection(
                "c1", [("foo", True, None)]
            )

        with self.assertRaises(InvalidArgumentType):
            ScanConfigs.modify_scan_config_set_family_selection(
                "c1", [("foo", "True", False)]
            )

        with self.assertRaises(InvalidArgument):
            ScanConfigs.modify_scan_config_set_family_selection(
                "c1", [("foo",)]
            )

    def test_modify_scan_config_set_name(self):
        request = ScanConfigs.modify_scan_config_set_name("c1", "foo")

        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="c1"><name>foo</name></modify_config>',
        )

    def test_modify_scan_config_set_name_missing_config_id(self):
        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_name(None, name="name")

        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_name("", name="name")

        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_name(config_id="", name="name")

    def test_modify_scan_config_set_name_missing_name(self):
        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_name("c1", None)

        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_name("c1", "")

    def test_modify_scan_config_set_nvt_preference(self):
        request = ScanConfigs.modify_scan_config_set_nvt_preference(
            "c1", nvt_oid="o1", name="foo"
        )

        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="c1">'
            b"<preference>"
            b'<nvt oid="o1"/>'
            b"<name>foo</name>"
            b"</preference>"
            b"</modify_config>",
        )

        request = ScanConfigs.modify_scan_config_set_nvt_preference(
            "c1", "foo", "o1"
        )

        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="c1">'
            b"<preference>"
            b'<nvt oid="o1"/>'
            b"<name>foo</name>"
            b"</preference>"
            b"</modify_config>",
        )

    def test_modify_scan_config_set_nvt_pref_with_value(self):
        request = ScanConfigs.modify_scan_config_set_nvt_preference(
            "c1", "foo", nvt_oid="o1", value="bar"
        )

        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="c1">'
            b"<preference>"
            b'<nvt oid="o1"/>'
            b"<name>foo</name>"
            b"<value>YmFy</value>"
            b"</preference>"
            b"</modify_config>",
        )

    def test_modify_scan_config_set_nvt_pref_missing_nvt_oid(self):
        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_nvt_preference(
                "c1", "foo", nvt_oid=None, value="bar"
            )

        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_nvt_preference(
                "c1", "foo", nvt_oid="", value="bar"
            )

        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_nvt_preference(
                "c1", "foo", "", value="bar"
            )

    def test_modify_scan_config_nvt_pref_missing_name(self):
        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_nvt_preference(
                "c1", name=None, nvt_oid="o1", value="bar"
            )

        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_nvt_preference(
                "c1", name="", nvt_oid="o1", value="bar"
            )

        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_nvt_preference(
                "c1", "", nvt_oid="o1", value="bar"
            )

    def test_modify_scan_config_set_nvt_preference_missing_config_id(self):
        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_nvt_preference("", "foo", "o1")

        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_nvt_preference(None, "foo", "o1")

    def test_modify_scan_config_set_nvt_selection(self):
        request = ScanConfigs.modify_scan_config_set_nvt_selection(
            "c1", "foo", ["o1"]
        )

        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="c1">'
            b"<nvt_selection>"
            b"<family>foo</family>"
            b'<nvt oid="o1"/>'
            b"</nvt_selection>"
            b"</modify_config>",
        )

        request = ScanConfigs.modify_scan_config_set_nvt_selection(
            "c1", "foo", ["o1", "o2"]
        )

        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="c1">'
            b"<nvt_selection>"
            b"<family>foo</family>"
            b'<nvt oid="o1"/>'
            b'<nvt oid="o2"/>'
            b"</nvt_selection>"
            b"</modify_config>",
        )

        request = ScanConfigs.modify_scan_config_set_nvt_selection(
            "c1", "foo", ("o1", "o2")
        )

        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="c1">'
            b"<nvt_selection>"
            b"<family>foo</family>"
            b'<nvt oid="o1"/>'
            b'<nvt oid="o2"/>'
            b"</nvt_selection>"
            b"</modify_config>",
        )

        request = ScanConfigs.modify_scan_config_set_nvt_selection(
            "c1", "foo", []
        )

        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="c1">'
            b"<nvt_selection>"
            b"<family>foo</family>"
            b"</nvt_selection>"
            b"</modify_config>",
        )

    def test_modify_scan_config_set_nvt_selection_missing_config_id(self):
        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_nvt_selection(
                None, "foo", ["o1"]
            )

        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_nvt_selection("", "foo", ["o1"])

    def test_modify_scan_config_set_nvt_selection_invalid_nvt_oids(self):
        with self.assertRaises(InvalidArgumentType):
            ScanConfigs.modify_scan_config_set_nvt_selection("c1", "foo", None)

        with self.assertRaises(InvalidArgumentType):
            ScanConfigs.modify_scan_config_set_nvt_selection("c1", "foo", "")

    def test_modify_scan_config_set_scanner_pref(self):
        request = ScanConfigs.modify_scan_config_set_scanner_preference(
            "c1", "foo"
        )

        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="c1">'
            b"<preference>"
            b"<name>foo</name>"
            b"</preference>"
            b"</modify_config>",
        )

    def test_modify_scan_config_set_scanner_pref_with_value(self):
        request = ScanConfigs.modify_scan_config_set_scanner_preference(
            "c1", "foo", value="bar"
        )

        self.assertEqual(
            bytes(request),
            b'<modify_config config_id="c1">'
            b"<preference>"
            b"<name>foo</name>"
            b"<value>YmFy</value>"
            b"</preference>"
            b"</modify_config>",
        )

    def test_modify_scan_config_scanner_pref_missing_name(self):
        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_scanner_preference(
                "c1", name=None, value="bar"
            )

        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_scanner_preference(
                "c1", name="", value="bar"
            )

        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_scanner_preference(
                "c1", "", value="bar"
            )

    def test_modify_scan_config_set_scanner_pref_missing_config_id(self):
        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_scanner_preference(
                None, name="foo"
            )

        with self.assertRaises(RequiredArgument):
            ScanConfigs.modify_scan_config_set_scanner_preference(
                "", name="foo"
            )



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from gvm.errors import InvalidArgument
from gvm.protocols.gmp.requests.v226 import FeedType


class GetFeedTypeFromStringTestCase(unittest.TestCase):
    def test_invalid(self):
        with self.assertRaises(InvalidArgument):
            FeedType.from_string("foo")

    def test_none_or_empty(self):
        ct = FeedType.from_string(None)
        self.assertIsNone(ct)
        ct = FeedType.from_string("")
        self.assertIsNone(ct)

    def test_nvt(self):
        ct = FeedType.from_string("nvt")
        self.assertEqual(ct, FeedType.NVT)

    def test_cert(self):
        ct = FeedType.from_string("cert")
        self.assertEqual(ct, FeedType.CERT)

    def test_scap(self):
        ct = FeedType.from_string("scap")
        self.assertEqual(ct, FeedType.SCAP)

    def test_gvmd_data(self):
        ct = FeedType.from_string("gvmd_data")
        self.assertEqual(ct, FeedType.GVMD_DATA)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import socketserver
import tempfile
import threading
import unittest
import uuid
from pathlib import Path
from unittest.mock import patch

from gvm.connections import (
    DEFAULT_TIMEOUT,
    DEFAULT_UNIX_SOCKET_PATH,
    GvmConnection,
    UnixSocketConnection,
)
from gvm.errors import GvmError


class DummyRequestHandler(socketserver.BaseRequestHandler):
    def handle(self):
        response = b'<gmp_response status="200" status_text="OK"/>'
        self.request.sendall(response)


class ThreadedUnixStreamServer(
    socketserver.ThreadingMixIn, socketserver.UnixStreamServer
):
    pass


class UnixSocketConnectionTestCase(unittest.TestCase):
    # pylint: disable=protected-access, invalid-name
    def setUp(self):
        self.socket_name = f"{tempfile.gettempdir()}/{uuid.uuid4()!s}.sock"
        self.socket_path = Path(self.socket_name)
        self.socket_server = ThreadedUnixStreamServer(
            self.socket_name, DummyRequestHandler
        )
        self.server_thread = threading.Thread(
            target=self.socket_server.serve_forever
        )
        self.server_thread.daemon = True
        self.server_thread.start()

    def tearDown(self):
        self.socket_server.server_close()
        self.socket_server.shutdown()
        self.server_thread.join(60.0)
        self.socket_path.unlink(missing_ok=True)

    def test_unix_socket_connection_connect_read(self):
        connection = UnixSocketConnection(
            path=self.socket_name, timeout=DEFAULT_TIMEOUT
        )
        connection.connect()
        resp = connection.read()
        self.assertEqual(resp, b'<gmp_response status="200" status_text="OK"/>')
        connection.disconnect()

    def test_unix_socket_connection_connect_send_bytes_read(self):
        connection = UnixSocketConnection(
            path=self.socket_name, timeout=DEFAULT_TIMEOUT
        )
        connection.connect()
        connection.send(b"<gmp/>")
        resp = connection.read()
        self.assertEqual(resp, b'<gmp_response status="200" status_text="OK"/>')
        connection.disconnect()

    def test_unix_socket_connect_file_not_found(self):
        connection = UnixSocketConnection(path="foo", timeout=DEFAULT_TIMEOUT)
        with self.assertRaises(GvmError, msg="Socket foo does not exist"):
            connection.connect()
        connection.disconnect()

    def test_unix_socket_connect_could_not_connect(self):
        connection = UnixSocketConnection(
            path=self.socket_name, timeout=DEFAULT_TIMEOUT
        )
        with patch("socket.socket.connect") as ConnectMock:  # noqa: N806
            connect_mock = ConnectMock
            connect_mock.side_effect = ConnectionError
            with self.assertRaises(
                GvmError, msg=f"Could not connect to socket {self.socket_name}"
            ):
                connection.connect()
            connection.disconnect()

    def test_unix_socket_send_unconnected_socket(self):
        connection = UnixSocketConnection(
            path=self.socket_name, timeout=DEFAULT_TIMEOUT
        )
        with self.assertRaises(GvmError, msg="Socket is not connected"):
            connection.send("<gmp>/")

    def test_init_no_args(self):
        connection = UnixSocketConnection()
        self.check_default_values(connection)

    def test_init_with_none(self):
        connection = UnixSocketConnection(path=None, timeout=None)
        self.check_default_values(connection)

    def check_default_values(self, connection: UnixSocketConnection):
        self.assertEqual(connection._timeout, DEFAULT_TIMEOUT)
        self.assertEqual(connection.path, DEFAULT_UNIX_SOCKET_PATH)

    def test_is_gvm_connection(self):
        connection = UnixSocketConnection()
        self.assertTrue(isinstance(connection, GvmConnection))



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest
from unittest.mock import MagicMock

import httpx

from gvm.protocols.http.openvasd._notus import NotusAPI


def _mock_response(status_code=200, json_data=None):
    response = MagicMock(spec=httpx.Response)
    response.status_code = status_code
    response.json.return_value = json_data or []
    response.raise_for_status = MagicMock()
    return response


class TestNotusAPI(unittest.TestCase):
    def setUp(self):
        self.mock_client = MagicMock(spec=httpx.Client)

    def test_get_os_list_success(self):
        mock_response = _mock_response(json_data=["debian", "alpine"])
        self.mock_client.get.return_value = mock_response

        api = NotusAPI(self.mock_client)
        result = api.get_os_list()
        self.mock_client.get.assert_called_once_with("/notus")
        mock_response.raise_for_status.assert_called_once()
        self.assertEqual(result, mock_response)

    def test_get_os_list_http_error_with_suppress_exceptions(self):
        mock_response = MagicMock()
        mock_response.status_code = 500

        self.mock_client.get.side_effect = httpx.HTTPStatusError(
            "Internal Server Error", request=MagicMock(), response=mock_response
        )

        api = NotusAPI(self.mock_client, suppress_exceptions=True)
        result = api.get_os_list()

        self.assertEqual(result, mock_response)

    def test_get_os_list_http_error(self):
        self.mock_client.get.side_effect = httpx.HTTPStatusError(
            "Error", request=MagicMock(), response=MagicMock(status_code=500)
        )

        api = NotusAPI(self.mock_client)
        with self.assertRaises(httpx.HTTPStatusError):
            api.get_os_list()

    def test_run_scan_success(self):
        mock_response = _mock_response(json_data={"vulnerabilities": []})
        self.mock_client.post.return_value = mock_response

        api = NotusAPI(self.mock_client)
        result = api.run_scan("debian", ["openssl", "bash"])
        self.mock_client.post.assert_called_once_with(
            "/notus/debian", json=["openssl", "bash"]
        )
        mock_response.raise_for_status.assert_called_once()
        self.assertEqual(result, mock_response)

    def test_run_scan_with_special_os_name(self):
        mock_response = _mock_response()
        self.mock_client.post.return_value = mock_response

        os_name = "alpine linux"
        encoded = "alpine%20linux"
        api = NotusAPI(self.mock_client)
        api.run_scan(os_name, ["musl"])
        self.mock_client.post.assert_called_once_with(
            f"/notus/{encoded}", json=["musl"]
        )

    def test_run_scan_http_error(self):
        self.mock_client.post.side_effect = httpx.HTTPStatusError(
            "Error", request=MagicMock(), response=MagicMock(status_code=400)
        )

        api = NotusAPI(self.mock_client)
        with self.assertRaises(httpx.HTTPStatusError):
            api.run_scan("ubuntu", ["curl"])

    def test_run_scan_http_error_with_suppress_exceptions(self):
        mock_response = MagicMock()
        mock_response.status_code = 500
        self.mock_client.post.side_effect = httpx.HTTPStatusError(
            "Internal Server Error", request=MagicMock(), response=mock_response
        )

        api = NotusAPI(self.mock_client, suppress_exceptions=True)
        response = api.run_scan("ubuntu", ["curl"])

        self.assertEqual(response, mock_response)



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_clone_policy import GmpClonePolicyTestMixin
from .test_create_policy import GmpCreatePolicyTestMixin
from .test_delete_policy import GmpDeletePolicyTestMixin
from .test_get_policies import GmpGetPoliciesTestMixin
from .test_get_policy import GmpGetPolicyTestMixin
from .test_import_policy import GmpImportPolicyTestMixin
from .test_modify_policy_set_comment import GmpModifyPolicySetCommentTestMixin
from .test_modify_policy_set_family_selection import (
    GmpModifyPolicySetFamilySelectionTestMixin,
)
from .test_modify_policy_set_name import GmpModifyPolicySetNameTestMixin
from .test_modify_policy_set_nvt_preference import (
    GmpModifyPolicySetNvtPreferenceTestMixin,
)
from .test_modify_policy_set_nvt_selection import (
    GmpModifyPolicySetNvtSelectionTestMixin,
)
from .test_modify_policy_set_scanner_preference import (
    GmpModifyPolicySetScannerPreferenceTestMixin,
)

__all__ = (
    "GmpClonePolicyTestMixin",
    "GmpCreatePolicyTestMixin",
    "GmpDeletePolicyTestMixin",
    "GmpGetPoliciesTestMixin",
    "GmpGetPolicyTestMixin",
    "GmpImportPolicyTestMixin",
    "GmpModifyPolicySetCommentTestMixin",
    "GmpModifyPolicySetFamilySelectionTestMixin",
    "GmpModifyPolicySetNameTestMixin",
    "GmpModifyPolicySetNvtPreferenceTestMixin",
    "GmpModifyPolicySetNvtSelectionTestMixin",
    "GmpModifyPolicySetScannerPreferenceTestMixin",
)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import GvmError


class GmpGetScanConfigTestMixin:
    def test_get_scan_config(self):
        self.gmp.get_scan_config("a1")

        self.connection.send.has_been_called_with(
            b'<get_configs config_id="a1" usage_type="scan" details="1"/>'
        )

    def test_get_scan_config_with_tasks(self):
        self.gmp.get_scan_config("a1", tasks=True)

        self.connection.send.has_been_called_with(
            b'<get_configs config_id="a1" usage_type="scan" '
            b'tasks="1" details="1"/>'
        )

    def test_get_scan_config_fail_without_scan_config_id(self):
        with self.assertRaises(GvmError):
            self.gmp.get_scan_config(None)

        with self.assertRaises(GvmError):
            self.gmp.get_scan_config("")



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import RequiredArgument
from gvm.protocols.gmp.requests.v224 import Nvts


class NvtsTestCase(unittest.TestCase):
    def test_get_nvt_families(self):
        request = Nvts.get_nvt_families()
        self.assertEqual(
            bytes(request),
            b"<get_nvt_families/>",
        )

    def test_get_nvt_families_with_sort_order(self):
        request = Nvts.get_nvt_families(sort_order="sort_order")
        self.assertEqual(
            bytes(request),
            b'<get_nvt_families sort_order="sort_order"/>',
        )

    def test_get_scan_config_nvts(self):
        request = Nvts.get_scan_config_nvts()
        self.assertEqual(
            bytes(request),
            b"<get_nvts/>",
        )

    def test_get_scan_config_nvts_with_details(self):
        request = Nvts.get_scan_config_nvts(details=True)
        self.assertEqual(
            bytes(request),
            b'<get_nvts details="1"/>',
        )

        request = Nvts.get_scan_config_nvts(details=False)
        self.assertEqual(
            bytes(request),
            b'<get_nvts details="0"/>',
        )

    def test_get_scan_config_nvts_with_preferences(self):
        request = Nvts.get_scan_config_nvts(preferences=True)
        self.assertEqual(
            bytes(request),
            b'<get_nvts preferences="1"/>',
        )

        request = Nvts.get_scan_config_nvts(preferences=False)
        self.assertEqual(
            bytes(request),
            b'<get_nvts preferences="0"/>',
        )

    def test_get_scan_config_nvts_with_preference_count(self):
        request = Nvts.get_scan_config_nvts(preference_count=True)
        self.assertEqual(
            bytes(request),
            b'<get_nvts preference_count="1"/>',
        )

        request = Nvts.get_scan_config_nvts(preference_count=False)
        self.assertEqual(
            bytes(request),
            b'<get_nvts preference_count="0"/>',
        )

    def test_get_scan_config_nvts_with_timeout(self):
        request = Nvts.get_scan_config_nvts(timeout=True)
        self.assertEqual(
            bytes(request),
            b'<get_nvts timeout="1"/>',
        )

        request = Nvts.get_scan_config_nvts(timeout=False)
        self.assertEqual(
            bytes(request),
            b'<get_nvts timeout="0"/>',
        )

    def test_get_scan_config_nvts_with_config_id(self):
        request = Nvts.get_scan_config_nvts(config_id="config_id")
        self.assertEqual(
            bytes(request),
            b'<get_nvts config_id="config_id"/>',
        )

    def test_get_scan_config_nvts_with_preferences_config_id(self):
        request = Nvts.get_scan_config_nvts(
            preferences_config_id="preferences_config_id"
        )
        self.assertEqual(
            bytes(request),
            b'<get_nvts preferences_config_id="preferences_config_id"/>',
        )

    def test_get_scan_config_nvts_with_family(self):
        request = Nvts.get_scan_config_nvts(family="family")
        self.assertEqual(
            bytes(request),
            b'<get_nvts family="family"/>',
        )

    def test_get_scan_config_nvts_with_sort_order(self):
        request = Nvts.get_scan_config_nvts(sort_order="sort_order")
        self.assertEqual(
            bytes(request),
            b'<get_nvts sort_order="sort_order"/>',
        )

    def test_get_scan_config_nvts_with_sort_field(self):
        request = Nvts.get_scan_config_nvts(sort_field="sort_field")
        self.assertEqual(
            bytes(request),
            b'<get_nvts sort_field="sort_field"/>',
        )

    def test_get_scan_config_nvt(self):
        request = Nvts.get_scan_config_nvt("nvt_oid")
        self.assertEqual(
            bytes(request),
            b'<get_nvts nvt_oid="nvt_oid" details="1" preferences="1" preference_count="1"/>',
        )

    def test_get_scan_config_nvt_missing_nvt_oid(self):
        with self.assertRaises(RequiredArgument):
            Nvts.get_scan_config_nvt(None)

        with self.assertRaises(RequiredArgument):
            Nvts.get_scan_config_nvt("")

    def test_get_nvts(self):
        request = Nvts.get_nvts()
        self.assertEqual(
            bytes(request),
            b'<get_info type="NVT"/>',
        )

    def test_get_nvts_with_name(self):
        request = Nvts.get_nvts(name="name")
        self.assertEqual(
            bytes(request),
            b'<get_info type="NVT" name="name"/>',
        )

    def test_get_nvts_with_details(self):
        request = Nvts.get_nvts(details=True)
        self.assertEqual(
            bytes(request),
            b'<get_info type="NVT" details="1"/>',
        )

        request = Nvts.get_nvts(details=False)
        self.assertEqual(
            bytes(request),
            b'<get_info type="NVT" details="0"/>',
        )

    def test_get_nvts_with_extended(self):
        request = Nvts.get_nvts(extended=True)
        self.assertEqual(
            bytes(request),
            b"<get_nvts/>",
        )

        request = Nvts.get_nvts(extended=False)
        self.assertEqual(
            bytes(request),
            b'<get_info type="NVT"/>',
        )

        request = Nvts.get_nvts(
            extended=False,
            details=True,
            preferences=True,  # gets ignored
            preference_count=True,  # gets ignored
            timeout=True,  # gets ignored
            config_id="config_id",  # gets ignored
            preferences_config_id="preferences_config_id",  # gets ignored
            family="family",  # gets ignored
            sort_order="sort_order",  # gets ignored
            sort_field="sort_field",  # gets ignored
        )
        self.assertEqual(bytes(request), b'<get_info type="NVT" details="1"/>')

        request = Nvts.get_nvts(
            extended=True,
            details=True,
            preferences=True,
            preference_count=True,
            timeout=True,
            config_id="config_id",
            preferences_config_id="preferences_config_id",
            family="family",
            sort_order="sort_order",
            sort_field="sort_field",
        )
        self.assertEqual(
            bytes(request),
            b'<get_nvts details="1" preferences="1" preference_count="1" timeout="1" config_id="config_id" preferences_config_id="preferences_config_id" family="family" sort_order="sort_order" sort_field="sort_field"/>',
        )

    def test_get_nvt(self):
        request = Nvts.get_nvt("nvt_id")
        self.assertEqual(
            bytes(request),
            b'<get_info info_id="nvt_id" type="NVT" details="1"/>',
        )

    def test_get_nvt_with_extended(self):
        request = Nvts.get_nvt("nvt_id", extended=True)
        self.assertEqual(
            bytes(request),
            b'<get_nvts nvt_oid="nvt_id" details="1" preferences="1" preference_count="1"/>',
        )

        request = Nvts.get_nvt("nvt_id", extended=False)
        self.assertEqual(
            bytes(request),
            b'<get_info info_id="nvt_id" type="NVT" details="1"/>',
        )

    def test_get_nvts_missing_nvt_id(self):
        with self.assertRaises(RequiredArgument):
            Nvts.get_nvt(None)

        with self.assertRaises(RequiredArgument):
            Nvts.get_nvt("")

    def test_get_nvt_preferences(self):
        request = Nvts.get_nvt_preferences()
        self.assertEqual(
            bytes(request),
            b"<get_preferences/>",
        )

    def test_get_nvt_preferences_with_nvt_oid(self):
        request = Nvts.get_nvt_preferences(nvt_oid="nvt_oid")
        self.assertEqual(
            bytes(request),
            b'<get_preferences nvt_oid="nvt_oid"/>',
        )

    def test_get_nvt_preference(self):
        request = Nvts.get_nvt_preference("preference")
        self.assertEqual(
            bytes(request),
            b'<get_preferences preference="preference"/>',
        )

    def test_get_nvt_preference_with_nvt_oid(self):
        request = Nvts.get_nvt_preference("preference", nvt_oid="nvt_oid")
        self.assertEqual(
            bytes(request),
            b'<get_preferences preference="preference" nvt_oid="nvt_oid"/>',
        )

    def test_get_nvt_preference_missing_name(self):
        with self.assertRaises(RequiredArgument):
            Nvts.get_nvt_preference(None)

        with self.assertRaises(RequiredArgument):
            Nvts.get_nvt_preference("")
